<?php
namespace Fiscal\Logic\NotaFiscalEletronica;

use Fiscal\Entities\Notafiscal;
use Geral\Entities\Endefili;
use Geral\Entities\Estado;
use Geral\Entities\Inscrestad;
use Geral\Entities\Lotenfe;
use Geral\Util\DateUtil;
use Indeterminado\Entities\Relacnflotenfe;
use Setforms2\Shared\Business\Notification;
use Setforms2\Shared\Notifier;
use Setforms2\Shared\Registry;
use Setforms2\Shared\Util\Util;
use Symfony\Component\Validator\Constraints\DateTime;
use Windows\WinPbi00157000\TabPbi00157000\Facade;

require_once('nfephp-master' . DIRECTORY_SEPARATOR . 'libs' . DIRECTORY_SEPARATOR . 'ToolsNFePHP.class.php');

/**
 * Created by JetBrains PhpStorm.
 * User: gustavoavila
 * Date: 07/01/13
 * Time: 08:58
 * To change this template use File | Settings | File Templates.
 */
class ToolsNFePHP extends \ToolsNFePHP
{

    const ID_LOTE_PROCESSADO = 104;
    const LOTE_PROCESSADO = 'P';
    const ID_LOTE_EM_PROCESSAMENTO = '105';
    const LOTE_EM_PROCESSAMENTO = 'E';
    const ID_NF_AUTORIZADA = 100;
    const ID_NF_AUTORIZADA_FORA_DO_PRAZO = 150;
    const ID_NF_DENEGADA = 110;
    const ID_NF_DENEGADA_EMIT = 301;
    const ID_NF_DENEGADA_DEST = 302;
    const NF_ACEITA = 'A';
    const NF_REJEITADA = 'R';
    const NF_DENEGADA = 'D';
    const BLOQUEADO = 'S';
    const DESBLOQUADO = 'N';
    const TIPO_END_PRINCIPAL = 'P';
    const FILIAL = 'L';
    const FORNECEDOR = 'F';
    const CLIENTE = 'C';
    const SAIDA = 'S';
    const ENTRADA = 'E';

    public $config = array();
    const ID_LOTE_NAO_LOCALIZADO = '106';
    const LOTE_NAO_LOCALIZADO = 'C';
    const PROBLEMA_RECIBO_OU_CNPJ = 'C';
    /**
     * URLPortal
     * Instância do WebService
     * @var string
     */
    private $URLPortal = 'http://www.portalfiscal.inf.br/nfe';

    /**
     * ####################### ATENCAO ##########################################################################
     * ########### NAO TROQUE QUALQUER VALOR DA CONFIGURACAO ####################################################
     * ########### SEM SABER O FUNCIONAMENTO DO MESMO ###########################################################
     * ########### Configurado para NFE de acordo com Manual de Orientacao Contribuinte v5.00 ###################
     * ##########################################################################################################
     */
    public function __construct(\Geral\Entities\Empresa $empresa, $cdinscesta)
    {
        //Inscricao estadual
        $inscrestad = self::getInscricaoEstadual($empresa->getCdempresa(), $empresa->getNrorg(), $cdinscesta);

        //###############################
        //########## GERAL ##############
        //###############################
        // tipo de ambiente esta informação deve ser editada pelo sistema
        // 1-Produção 2-Homologação
        // esta variável será utilizada para direcionar os arquivos e
        // estabelecer o contato com o SEFAZ
        $this->config["ambiente"] = $inscrestad->getIdtpambientenfe();
        //esta variável contêm o nome do arquivo com todas as url dos webservices do sefaz
        //incluindo a versao dos mesmos, pois alguns estados não estão utilizando as
        //mesmas versões
        $this->config["arquivoURLxml"]    = "nfe_ws2.xml";
        $this->config["arquivoURLxmlCTe"] = "cte_ws1.xml";
        //Diretório onde serão mantidos os arquivos com as NFe em xml
        //a partir deste diretório serão montados todos os subdiretórios do sistema
        //de manipulação e armazenamento das NFe e CTe
        $cnpj = trim(\Geral\Util\Util::removeMascaraInscricao($empresa->getNrinsjurempr()));
        $this->config["arquivosDir"] = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'notasfiscais' . DIRECTORY_SEPARATOR . $empresa->getNrorg() . DIRECTORY_SEPARATOR . $cnpj;
        $this->config["arquivosDirCTe"]    = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'notasfiscais' . DIRECTORY_SEPARATOR . $empresa->getNrorg() . DIRECTORY_SEPARATOR . $cnpj;
        $this->config["caminhoPDFreports"] = APP_PATH . DIRECTORY_SEPARATOR . 'public' . DIRECTORY_SEPARATOR . 'reports' . DIRECTORY_SEPARATOR;
        //URL base da API, passa a ser necessária em virtude do uso dos arquivos wsdl
        //para acesso ao ambiente nacional
        $this->config["baseurl"] = "http://localhost:9090/nfephp";
        //Versão em uso dos shemas utilizados para validação dos xmls
        $this->config["schemes"]    = "PL_006q";
        $this->config["schemesCTe"] = "PL_CTE_104";

        //###############################
        //###### EMPRESA EMITENTE #######
        //###############################
        //Nome da Empresa
        $this->config["empresa"] = $empresa->getNmrazsocempr();
        $estado                  = Estado::findOneByCdpaisAndSgestado($empresa->getCdpais(), $empresa->getSgestado());
        //Sigla da UF
        $this->config["UF"] = $estado->getSgestado();
        //Código da UF
        $this->config["cUF"] = $estado->getCdestadoibge();
        //Número do CNPJ
        $this->config["cnpj"] = trim(\Geral\Util\Util::removeMascaraInscricao($empresa->getNrinsjurempr()));

        //Número da Organização
        $this->config['nrorg'] = $empresa->getNrorg();

        //###############################
        //#### CERITIFICADO DIGITAL #####
        //###############################
        //        $jsonDataset = new JsonDataset(Facade::getFullFilePath());
        //        $dataset = $jsonDataset->find(array("CDEMPRESA" => $empresa->getCdempresa(),  "CNPJ" => $empresa->getNrinsjurempr()));
        if (!$inscrestad->getNmcertificado()) {
            throw new \Exception("Certificado não cadastrado.");
        }
        $this->config["cdinscesta"] = $cdinscesta;
        //        $dadosCertific = $dataset->first();
        //Nome do certificado que deve ser colocado na pasta certs da API
        $this->config["certName"] = $inscrestad->getNmcertificado() . '.pfx';
        //Senha da chave privada
        $this->config["keyPass"] = Util::decrypt($inscrestad->getCdsenhacertif());
        //Senha de decriptaçao da chave, normalmente não é necessaria
        $this->config["passPhrase"] = "";

        //###############################
        //############ DANFE ############
        //###############################
        //Configuração do DANFE
        $this->config["danfeFormato"] = "P"; //P-Retrato L-Paisagem
        $this->config["danfePapel"]   = "A4"; //Tipo de papel utilizado
        $this->config["danfeCanhoto"] = 1; //se verdadeiro imprime o canhoto na DANFE
        $this->config["danfeLogo"]    = "/var/www/nfephp/images/logo.jpg"; //passa o caminho para o LOGO da empresa
        $this->config["danfeLogoPos"] = "L"; //define a posição do logo na Danfe L-esquerda, C-dentro e R-direta
        $this->config["danfeFonte"]   = "Times"; //define a fonte do Danfe limitada as fontes compiladas no FPDF (Times)
        $this->config["danfePrinter"] = "hpteste"; //define a impressora para impressão da Danfe

        //###############################
        //############ DACTE ############
        //###############################
        //Configuração do DACTE
        $this->config["dacteFormato"] = "P"; //P-Retrato L-Paisagem
        $this->config["dactePapel"]   = "A4"; //Tipo de papel utilizado
        $this->config["dacteCanhoto"] = 1; //se verdadeiro imprime o canhoto na DANFE
        $this->config["dacteLogo"]    = "/var/www/nfephp/images/logo.jpg"; //passa o caminho para o LOGO da empresa
        $this->config["dacteLogoPos"] = "L"; //define a posição do logo na Danfe L-esquerda, C-dentro e R-direta
        $this->config["dacteFonte"]   = "Times"; //define a fonte do Danfe limitada as fontes compiladas no FPDF (Times)
        $this->config["dactePrinter"] = "hpteste"; //define a impressora para impressão da Dacte

        //###############################
        //############ EMAIL ############
        //###############################
        //Configuração do email
        $this->config["mailAuth"]         = "1"; //ativa ou desativa a obrigatoriedade de autenticação no envio de email, na maioria das vezes ativar
        $this->config["mailFROM"]         = "nfe@seudominio.com.br"; //identificação do emitente
        $this->config["mailHOST"]         = "smtp.seudominio.com.br"; //endereço do servidor SMTP
        $this->config["mailUSER"]         = "nfe@seudominio.com.br"; //username para autenticação, usando quando mailAuth é 1
        $this->config["mailPASS"]         = "suasenha"; //senha de autenticação do serviço de email
        $this->config["mailPROTOCOL"]     = ""; //protocolo de email utilizado (classe alternate)
        $this->config["mailPORT"]         = "25"; //porta utilizada pelo smtp (classe alternate)
        $this->config["mailFROMmail"]     = "nfe@seudominio.com.br"; //para alteração da identificação do remetente, pode causar problemas com filtros de spam
        $this->config["mailFROMname"]     = "NFe"; //para indicar o nome do remetente
        $this->config["mailREPLYTOmail"]  = "nfe@seudominio.com.br"; //para indicar o email de resposta
        $this->config["mailREPLYTOname"]  = "NFe"; //para indicar email de cópia
        $this->config["mailIMAPhost"]     = "mail.seudominio.com.br"; //url para o servidor IMAP
        $this->config["mailIMAPport"]     = "143"; //porta do servidor IMAP
        $this->config["mailIMAPsecurity"] = "tls"; //esquema de segurança do servidor IMAP
        $this->config["mailIMAPnocerts"]  = "novalidate-cert"; //desabilita verificação de certificados do Servidor IMAP
        $this->config["mailIMAPbox"]      = "INBOX"; //caixa postal de entrada do servidor IMAP
        $this->config["mailLayoutFile"]   = ""; //layout da mensagem do email

        //###############################
        //############ PROXY ############
        //###############################
        //Configuração de Proxy
        $this->config["proxyIP"]   = ""; //ip do servidor proxy, se existir
        $this->config["proxyPORT"] = ""; //numero da porta usada pelo proxy
        $this->config["proxyUSER"] = ""; //nome do usuário, se o proxy exigir autenticação
        $this->config["proxyPASS"] = ""; //senha de autenticação do proxy

        $this->certsDir = $empresa->getNrorg() . DIRECTORY_SEPARATOR . trim(\Geral\Util\Util::removeMascaraInscricao($empresa->getNrinsjurempr())) . DIRECTORY_SEPARATOR;

        parent::__construct($this->config, 2, true);

    }

    /**
     * Retorna inscricao estadual da empresa
     * @param $cdempresa
     * @param $cdinscestad
     * @param $nrorg
     * @return Inscrestad
     * @throws \Exception
     */
    private static function getInscricaoEstadual($cdempresa, $nrorg, $cdinscesta)
    {
        $inscrestad = Inscrestad::findOneByCdempresaAndNrorgAndCdinscesta($cdempresa, $nrorg, $cdinscesta);

        if (!($inscrestad instanceof Inscrestad)) {
            throw new \Exception('Inscrição Estadual não Encontrada');
        }

        return $inscrestad;
    }

    /**
     * autoEnv
     * Este método procura por NFe's na pasta VALIDADAS, se houver alguma envia para a SEFAZ
     * e em saso de sucesso no envio move o arquivo para a pasta das enviadas
     * ATENÇÃO : Existe um limite para o tamanho do arquivo a ser enviado ao SEFAZ
     *
     * @version 2.0
     * @param $nrlote : Número do Lote
     * @return boolean TRUE sucesso ou FALSE erro
     */
    public function autoEnvNFe($nrlote)
    {
        //varre a pasta de validadas
        $aName = $this->listDir($this->valDir, '*-nfe.xml', false);
        //se houver algum arquivo *-nfe.xml continua, caso contrario sai
        $n = count($aName);
        if ($n > 0) {
            //determina o numero de grupos de envio com 10 notas por grupo
            $k = ceil($n / 10);
            // as notas localizadas na pasta validadas serão enviadas em
            // grupos de até 10 notas de cada vez, sim este é um valor arbritrario eu quis até 10
            for ($i = 0; $i < $k; $i++) {
                //limpa a matriz com as notas fiscais
                $aNFE = null;
                for ($x = $i * 10; $x < (($i + 1) * 10); $x++) {
                    if ($x < $n) {
                        $filename = $this->valDir . $aName[$x];
                        $nfefile  = file_get_contents($filename);
                        $aNFE[]   = $nfefile;
                    }
                }
                //enviar as notas
                if ($ret = $this->sendLot($aNFE, $nrlote)) {
                    //incrementa o numero do lote no controle
                    if ($this->__putNumLot($nrlote)) {
                        $this->errStatus = true;
                        $this->errMsg .= ' Falha na Gravação do numero do lote de envio!! ';
                        return false;
                    }
                    //mover as notas do lote para o diretorio de enviadas
                    //para cada em $aNames[] mover para $this->envDir
                    for ($x = $i * 10; $x < (($i + 1) * 10); $x++) {
                        if ($x < $n) {
                            if (!rename($this->valDir . $aName[$x], $this->envDir . $aName[$x])) {
                                $this->errStatus = true;
                                $this->errMsg .= ' Falha na movimentação da NFe das "validadas" para "enviadas"!! ';
                                return false;
                            }
                        }
                    } //fim for rename
                    //incrementa o numero
                } else {
                    $this->errStatus = true;
                    $this->errMsg .= ' Erro no envio do lote de NFe!! ';
                    return false;
                }
            }
        } else {
            return false;
        }
        return true;
    }

    /**
     * sendLot
     * Envia lote de Notas Fiscais para a SEFAZ.
     * Este método pode enviar uma ou mais NFe para o SEFAZ, desde que,
     * o tamanho do arquivo de envio não ultrapasse 500kBytes
     * Este processo enviará somente até 50 NFe em cada Lote
     *
     * @name             sendLot
     * @version 2.1.11
     * @package NFePHP
     * @author  Roberto L. Machado <linux.rlm at gmail dot com>
     * @param    mixed   $mNFe    string com uma nota fiscal em xml ou um array com as NFe em xml, uma em cada campo do array unidimensional MAX 50
     * @param   integer  $idLote  id do lote e um numero que deve ser gerado pelo sistema
     *                            a cada envio mesmo que seja de apenas uma NFe
     * @param int|string $modSOAP 1 usa __sendSOP e 2 usa __sendSOAP2
     * @throws \Exception
     * @return    mixed    false ou array ['bStat'=>false,'cStat'=>'','xMotivo'=>'','dhRecbto'=>'','nRec'=>'','tMed'=>'','tpAmb'=>'','verAplic'=>'','cUF'=>'']
     * @todo    Incluir regra de validação para ambiente de homologação/produção vide NT2011.002
     */
    public function sendLot($mNFe, $idLote, $metodoTransmissao = 'SEFAZ', $modSOAP = '2')
    {
        //variavel de retorno do metodo
        $aRetorno = array('bStat' => false, 'cStat' => '', 'xMotivo' => '', 'dhRecbto' => '', 'nRec' => '', 'tMed' => '', 'tpAmb' => '', 'verAplic' => '', 'cUF' => '');
        /*//verifica se o SCAN esta habilitado
        if (!$this->enableSCAN){
            $aURL = $this->aURL;
        } else {
            //Alterado. Urls do webservice agora são armazenadas no banco.
            $aURL = $this->loadSEFAZ( $this->raizDir . 'config' . DIRECTORY_SEPARATOR . $this->xmlURLfile,$this->config["ambiente"],$this->config["UF"], true);
        }*/
        if ($metodoTransmissao == 'SEFAZ') {
            $aURL = $this->aURL;
        } elseif ($metodoTransmissao == 'SVAN') {
            $aURL = $this->loadSEFAZ($this->raizDir . 'config' . DIRECTORY_SEPARATOR . $this->xmlURLfile, $this->config["ambiente"], $this->config["UF"], true);
        }
        //identificação do serviço
//        $servico = 'NfeRecepcao';
        $servico = 'NfeAutorizacao';
        //recuperação da versão
        $versao = $aURL[$servico]['version'];
        //recuperação da url do serviço
        $urlservico = $aURL[$servico]['URL'];
        //recuperação do método
        $metodo = $aURL[$servico]['method'];
        //montagem do namespace do serviço
//        $namespace = $this->URLPortal . '/wsdl/' . $servico . '2';
        $namespace = $this->URLPortal . '/wsdl/' . $servico;
        // limpa a variavel
        $sNFe = '';
        if (is_array($mNFe)) {
            // verificar se foram passadas até 50 NFe
            if (count($mNFe) > 50) {
                throw new \Exception("No maximo 50 NFe devem compor um lote de envio!!");
            }
            // monta string com todas as NFe enviadas no array
            $sNFe = implode('', $mNFe);
        } else {
            $sNFe = $mNFe;
        }
        //remover <?xml version="1.0" encoding=... das NFe pois somente uma dessas tags pode exitir na mensagem
        $sNFe = str_replace(array('<?xml version="1.0" encoding="utf-8"?>', '<?xml version="1.0" encoding="UTF-8"?>'), '', $sNFe);
        $sNFe = str_replace(array("\r", "\n", "\s"), "", $sNFe);
        // Monta tag do lote enviNFE
        $enviNFE = '<enviNFe xmlns="' . $this->URLPortal . '" versao="' . $versao . '"><idLote>' . $idLote . '</idLote><indSinc>0</indSinc>' . $sNFe . '</enviNFe>';
        //montagem do cabeçalho da comunicação SOAP
        $cabec = '<nfeCabecMsg xmlns="' . $namespace . '"><cUF>' . $this->config["cUF"] . '</cUF><versaoDados>' . $versao . '</versaoDados></nfeCabecMsg>';
        //montagem dos dados da mensagem SOAP
        $dados = '<nfeDadosMsg xmlns="' . $namespace . '">' . $enviNFE . '</nfeDadosMsg>';
        //envia dados via SOAP
        if ($modSOAP == '2') {
            $retorno = $this->__sendSOAP2($urlservico, $namespace, $cabec, $dados, $metodo, $this->config["ambiente"]);
        } else {
            $retorno = $this->__sendSOAP($urlservico, $namespace, $cabec, $dados, $metodo, $this->config["ambiente"], $this->config["UF"]);
        }
        //grava lote enviado.
        file_put_contents($this->envDir . $idLote . '-env-lot.xml', $enviNFE);
        //verifica o retorno
        if ($retorno) {
            //tratar dados de retorno
            $doc                     = new \DOMDocument('1.0', 'utf-8'); //cria objeto DOM
            $doc->formatOutput       = false;
            $doc->preserveWhiteSpace = false;
            $doc->loadXML($retorno, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
            $cStat = !empty($doc->getElementsByTagName('cStat')->item(0)->nodeValue) ? $doc->getElementsByTagName('cStat')->item(0)->nodeValue : '';
            if ($cStat == '') {
                //houve erro
                throw new \Exception("Falha na comunicação SOAP ! O retorno não contêm cStat !");
            } else {
                if ($cStat == '103') {
                    $aRetorno['bStat'] = true;
                }
            }
            // status do serviço
            $aRetorno['cStat'] = $doc->getElementsByTagName('cStat')->item(0)->nodeValue;
            // motivo da resposta (opcional)
            $aRetorno['xMotivo'] = !empty($doc->getElementsByTagName('xMotivo')->item(0)->nodeValue) ? $doc->getElementsByTagName('xMotivo')->item(0)->nodeValue : '';
            // data e hora da mensagem (opcional)
            $aRetorno['dhRecbto'] = !empty($doc->getElementsByTagName('dhRecbto')->item(0)->nodeValue) ? date("d/m/Y H:i:s", $this->__convertTime($doc->getElementsByTagName('dhRecbto')->item(0)->nodeValue)) : '';
            // numero do recibo do lote enviado (opcional)
            $aRetorno['nRec'] = !empty($doc->getElementsByTagName('nRec')->item(0)->nodeValue) ? $doc->getElementsByTagName('nRec')->item(0)->nodeValue : '';
            $this->nrrecibo   = $aRetorno['nRec'];
            //outras informações
            $aRetorno['tMed']     = !empty($doc->getElementsByTagName('tMed')->item(0)->nodeValue) ? $doc->getElementsByTagName('tMed')->item(0)->nodeValue : '';
            $aRetorno['tpAmb']    = !empty($doc->getElementsByTagName('tpAmb')->item(0)->nodeValue) ? $doc->getElementsByTagName('tpAmb')->item(0)->nodeValue : '';
            $aRetorno['verAplic'] = !empty($doc->getElementsByTagName('verAplic')->item(0)->nodeValue) ? $doc->getElementsByTagName('verAplic')->item(0)->nodeValue : '';
            $aRetorno['cUF']      = !empty($doc->getElementsByTagName('cUF')->item(0)->nodeValue) ? $doc->getElementsByTagName('cUF')->item(0)->nodeValue : '';
            //gravar o retorno na pasta temp
            $nome = $this->temDir . $idLote . '-rec.xml';
            $nome = $doc->save($nome);
        } else {
            throw new \Exception("Nao houve retorno da comunicação SOAP !");
        }
        return $aRetorno;
    } // fim autoSendNFe

    /**
     * Solicita inutilizaçao de uma serie de numeros de NF
     *
     * @name inutNF
     * @version 2.1
     * @package NFePHP
     * @param    string $nAno       ano com 2 digitos
     * @param   string $nSerie     serie da NF 1 até 3 digitos
     * @param int|string $nIni numero inicial 1 até 9 digitos zero a esq
     * @param int|string $nFin numero Final 1 até 9 digitos zero a esq
     * @param   string $xJust      justificativa 15 até 255 digitos
     * @param int|string $modSOAP 1 usa __sendSOAP e 2 usa __sendSOAP2
     * @return    mixed False ou array ['bStat'=>boolean,'cStat'=>'','xMotivo'=>'','dhRecbto'=>'','nProt'=>'']
     */
    public function inutNF($nAno='',$nSerie='1',$nIni='',$nFin='',$xJust='',$modSOAP='2'){

        //variavel de retorno do metodo
        $aRetorno = array('bStat'=>false,'cStat'=>'','xMotivo'=>'','dhRecbto'=>'','nProt'=>'');

        //verifica se o SCAN esta habilitado
        //        if (!$this->enableSCAN){
        $aURL = $this->aURL;
        //        } else {
        //            $aURL = $this->loadSEFAZ( $this->raizDir . 'config' . DIRECTORY_SEPARATOR . "def_ws2.xml",$tpAmb,'SCAN');
        //        }

        // valida o campo ano
        if( strlen($nAno) > 2 ){
            $this->errStatus = true;
            $this->errMsg = 'O ano tem mais de 2 digitos. Corrija e refaça o processo!!';
            return false;
        } else {
            if (strlen($nAno) < 2 ){
                $this->errStatus = true;
                $this->errMsg = 'O ano tem menos de 2 digitos. Corrija e refaça o processo!!';
                return false;
            }
        }

        //valida o campo serie
        if( strlen($nSerie) == 0 || strlen($nSerie) > 3){
            $this->errStatus = true;
            $this->errMsg = "O campo serie está errado: $nSerie. Corrija e refaça o processo!!";
            return false;
        }

        //valida o campo numero inicial
        if (strlen($nIni) < 1 || strlen($nIni) > 9){
            $this->errStatus = true;
            $this->errMsg = "O campo numero inicial está errado: $nIni. Corrija e refaça o processo!!";
            return false;
        }

        //valida o campo numero final
        if (strlen($nFin) < 1 || strlen($nFin) > 9){
            $this->errStatus = true;
            $this->errMsg = "O campo numero final está errado: $nFin. Corrija e refaça o processo!!";
            return false;
        }

        //valida o campo justificativa
        $nL = strlen($xJust);
        if ($nL < 15 ) {
            $this->errStatus = true;
            $this->errMsg = "A justificativa é menor que o permitido, apenas $nL letras. Corrija e refaça o processo!!";
            return false;
        } else {
            if ($nL > 255 ) {
                $this->errStatus = true;
                $this->errMsg = "A justificativa é maior que o permitido, $nL letras, no mácximo podem ser 255. Corrija e refaça o processo!!";
                return false;
            }
        }

        //identificação do serviço
        $servico = 'NfeInutilizacao';
        //recuperação da versão
        $versao = $aURL[$servico]['version'];
        //recuperação da url do serviço
        $urlservico = $aURL[$servico]['URL'];
        //recuperação do método
        $metodo = $aURL[$servico]['method'];
        //montagem do namespace do serviço
        $namespace = $this->URLPortal.'/wsdl/'.$servico.'2';

        //Identificador da TAG a ser assinada formada com Código da UF +
        //Ano (2 posições) + CNPJ + modelo + série + nro inicial e nro final
        //precedida do literal “ID”
        // 43 posições
        //     2      4       6       20      22    25       34      43
        //     2      2       2       14       2     3        9       9
        $id = 'ID'.$this->cUF.$nAno.$this->cnpj.'55'.str_pad($nSerie,3,'0',STR_PAD_LEFT).str_pad($nIni,9,'0',STR_PAD_LEFT).str_pad($nFin,9,'0',STR_PAD_LEFT);

        //montagem do cabeçalho da comunicação SOAP
        $cabec = '<nfeCabecMsg xmlns="'.$namespace.'"><cUF>'.$this->cUF.'</cUF><versaoDados>'.$versao.'</versaoDados></nfeCabecMsg>';
        //montagem do corpo da mensagem
//        $dXML = '<inutNFe xmlns="'.$this->URLPortal.'" xmlns:xsi="'.$this->URLxsi.'" xmlns:xsd="'.$this->URLxsd.'" versao="'.$versao.'">';
        $dXML = '<inutNFe xmlns="'.$this->URLPortal.'" versao="'.$versao.'">';
        $dXML .= '<infInut Id="'.$id.'">';
        $dXML .= '<tpAmb>'.$this->tpAmb.'</tpAmb>';
        $dXML .= '<xServ>INUTILIZAR</xServ>';
        $dXML .= '<cUF>'.$this->cUF.'</cUF>';
        $dXML .= '<ano>'.$nAno.'</ano>';
        $dXML .= '<CNPJ>'.$this->cnpj.'</CNPJ>';
        $dXML .= '<mod>55</mod>';
        $dXML .= '<serie>'.$nSerie.'</serie>';
        $dXML .= '<nNFIni>'.$nIni.'</nNFIni>';
        $dXML .= '<nNFFin>'.$nFin.'</nNFFin>';
        $dXML .= '<xJust>'.$xJust.'</xJust>';
        $dXML .= '</infInut>';
        $dXML .= '</inutNFe>';
        //assina a lsolicitação de inutilização
        $dXML = $this->signXML($dXML,'infInut');
        $dados = '<nfeDadosMsg xmlns="'.$namespace.'">'.$dXML.'</nfeDadosMsg>';
        //remove as tags xml que porventura tenham sido inclusas
        $dados = str_replace('<?xml version="1.0"?>','', $dados);
        $dados = str_replace('<?xml version="1.0" encoding="utf-8"?>','', $dados);
        //envia a solicitação via SOAP
        if ($modSOAP == '2'){
            $retorno = $this->__sendSOAP2($urlservico, $namespace, $cabec, $dados, $metodo, $this->config["ambiente"]);
        } else {
            $retorno = $this->__sendSOAP($urlservico, $namespace, $cabec, $dados, $metodo, $this->config["ambiente"], $this->config["UF"]);
        }
        //verifica o retorno
        if ($retorno){
            //tratar dados de retorno
            $doc = new \DOMDocument(); //cria objeto DOM
            $doc->formatOutput = false;
            $doc->preserveWhiteSpace = false;
            $doc->loadXML($retorno,LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
            $cStat = !empty($doc->getElementsByTagName('cStat')->item(0)->nodeValue) ? $doc->getElementsByTagName('cStat')->item(0)->nodeValue : '';
            if ($cStat == ''){
                //houve erro
                return false;
            } else {
                //verificar o status da solicitação
                if ($cStat == '102'){
                    $aRetorno['bStat'] = true;
                }
            }
            // status do serviço se 102 inutilização aceita
            $aRetorno['cStat'] = $doc->getElementsByTagName('cStat')->item(0)->nodeValue;
            // motivo da resposta (opcional)
            $aRetorno['xMotivo'] = !empty($doc->getElementsByTagName('xMotivo')->item(0)->nodeValue) ? $doc->getElementsByTagName('xMotivo')->item(0)->nodeValue : '';
            // data e hora da mensagem (opcional)
            $aRetorno['dhRecbto'] = !empty($doc->getElementsByTagName('dhRecbto')->item(0)->nodeValue) ? date("d/m/Y H:i",$this->__convertTime($doc->getElementsByTagName('dhRecbto')->item(0)->nodeValue)) : '';
            // numero do protocolo de aceitação da inutilização (opcional)
            $aRetorno['nProt'] = !empty($doc->getElementsByTagName('nProt')->item(0)->nodeValue) ? $doc->getElementsByTagName('nProt')->item(0)->nodeValue : '';
            //gravar o retorno na pasta temp
            $nome = $this->temDir.$id.'-inut.xml';
            $nome = $doc->save($nome);
        } else {
            $this->errStatus = true;
            $this->errMsg = 'Nao houve retorno Soap verifique a mensagem de erro e o debug!!';
            $aRetorno = false;
        }
        return $aRetorno;

    } //fim inutNFe


    public function loadSEFAZ($spathXML, $tpAmb, $sUF, $svan = false)
    {
        /** @var $webservice \Fiscal\Entities\Webservice */
        $webservice = \Fiscal\Entities\Webservice::findOneBySgestadoAndIdambienteAndIdservico($sUF, $tpAmb, self::SERVICO);

        if ($svan) {
            //extrai a variável cUF do lista
            //            $this->cUF = $this->cUFlist['SVAN'];
            $this->modoSerie = $webservice->getCdsvc();

            /** @var  $webservicesvc \Fiscal\Entities\Webservicesvc */
            $webservicesvc = \Fiscal\Entities\Webservicesvc::findOneByCdsvcAndIdambiente($webservice->getCdsvc(), $tpAmb);

            $aUrl['NfeRecepcao']['URL']          = $webservicesvc->getDswsrecepcaosvc();
            $aUrl['NfeRecepcao']['method']       = self::retornaMetodoUrl($webservicesvc->getDswsrecepcaosvc());
//            $aUrl['NfeRecepcao']['version']      = '2.00';
            $aUrl['NfeRecepcao']['version']      = '3.10';
            $aUrl['NfeRetRecepcao']['URL']       = $webservicesvc->getDswsretrecepcaosvc();
            $aUrl['NfeRetRecepcao']['method']    = self::retornaMetodoUrl($webservicesvc->getDswsretrecepcaosvc());
//            $aUrl['NfeRetRecepcao']['version']   = '2.00';
            $aUrl['NfeRetRecepcao']['version']   = '3.10';
            $aUrl['RecepcaoEvento']['URL']       = $webservicesvc->getDswseventosvc();
            $aUrl['RecepcaoEvento']['method']    = self::retornaMetodoUrl($webservicesvc->getDswseventosvc());
            $aUrl['RecepcaoEvento']['version']   = '1.00';
            $aUrl['NfeStatusServico']['URL']     = $webservicesvc->getDswsstatusservicosvc();
            $aUrl['NfeStatusServico']['method']  = self::retornaMetodoUrl($webservicesvc->getDswsstatusservicosvc());
//            $aUrl['NfeStatusServico']['version'] = '2.00';
            $aUrl['NfeStatusServico']['version'] = '3.10';
            $aUrl['NfeConsulta']['URL']          = $webservicesvc->getDswsconsultaprosvc();
            $aUrl['NfeConsulta']['method']       = self::retornaMetodoUrl($webservicesvc->getDswsconsultaprosvc());
//            $aUrl['NfeConsulta']['version']      = '2.01';
            $aUrl['NfeConsulta']['version']      = '3.10';
            $aUrl['NfeAutorizacao']['URL']       = $webservicesvc->getDswsautorizacaosvc();
            $aUrl['NfeAutorizacao']['method']    = self::retornaMetodoUrl($webservicesvc->getDswsautorizacaosvc());
            $aUrl['NfeAutorizacao']['version']   = '3.10';
            $aUrl['NfeRetAutorizacao']['URL']    = $webservicesvc->getDswsretautorizacaosvc();
            $aUrl['NfeRetAutorizacao']['method'] = self::retornaMetodoUrl($webservicesvc->getDswsretautorizacaosvc());
            $aUrl['NfeRetAutorizacao']['version']= '3.10';

            return $aUrl;
        }

        //extrai a variável cUF do lista
        $this->cUF = $this->cUFlist[$sUF];

        $aUrl['NfeRecepcao']['URL']             = $webservice->getWsrecepcao();
        $aUrl['NfeRecepcao']['method']          = self::retornaMetodoUrl($webservice->getWsrecepcao());
//        $aUrl['NfeRecepcao']['version']         = '2.00';
        $aUrl['NfeRecepcao']['version']         = '3.10';
        $aUrl['NfeRetRecepcao']['URL']          = $webservice->getWsretrecepcao();
        $aUrl['NfeRetRecepcao']['method']       = self::retornaMetodoUrl($webservice->getWsretrecepcao());
//        $aUrl['NfeRetRecepcao']['version']      = '2.00';
        $aUrl['NfeRetRecepcao']['version']      = '3.10';
        $aUrl['NfeInutilizacao']['URL']         = $webservice->getWsinutilizacao();
        $aUrl['NfeInutilizacao']['method']      = self::retornaMetodoUrl($webservice->getWsinutilizacao());
//        $aUrl['NfeInutilizacao']['version']     = '2.00';
        $aUrl['NfeInutilizacao']['version']     = '3.10';
        $aUrl['NfeConsulta']['URL']             = $webservice->getWsconsultapro();
        $aUrl['NfeConsulta']['method']          = self::retornaMetodoUrl($webservice->getWsconsultapro());
//        $aUrl['NfeConsulta']['version']         = '2.01';
        $aUrl['NfeConsulta']['version']         = '3.10';
        $aUrl['NfeStatusServico']['URL']        = $webservice->getWsstatusservico();
        $aUrl['NfeStatusServico']['method']     = self::retornaMetodoUrl($webservice->getWsstatusservico());
//        $aUrl['NfeStatusServico']['version']    = '2.00';
        $aUrl['NfeStatusServico']['version']    = '3.10';
        $aUrl['CadConsultaCadastro']['URL']     = $webservice->getWsconsultacad();
        $aUrl['CadConsultaCadastro']['method']  = self::retornaMetodoUrl($webservice->getWsconsultacad());
        $aUrl['CadConsultaCadastro']['version'] = '2.00';
        $aUrl['RecepcaoEvento']['URL']          = $webservice->getWsevento();
        $aUrl['RecepcaoEvento']['method']       = self::retornaMetodoUrl($webservice->getWsevento());
        $aUrl['RecepcaoEvento']['version']      = '1.00';
        $aUrl['NfeConsultaDest']['URL']         = $webservice->getWsconsultadest();
        $aUrl['NfeConsultaDest']['method']      = self::retornaMetodoUrl($webservice->getWsconsultadest());
        $aUrl['NfeConsultaDest']['version']     = '1.00';
        $aUrl['NfeDownloadNF']['URL']           = $webservice->getWsdownloadnf();
        $aUrl['NfeDownloadNF']['method']        = self::retornaMetodoUrl($webservice->getWsdownloadnf());
        $aUrl['NfeDownloadNF']['version']       = '1.00';

        $aUrl['NfeAutorizacao']['URL']       = $webservice->getWsautorizacao();
        $aUrl['NfeAutorizacao']['method']    = self::retornaMetodoUrl($webservice->getWsautorizacao());
        $aUrl['NfeAutorizacao']['version']   = '3.10';
        $aUrl['NfeRetAutorizacao']['URL']    = $webservice->getWsretautorizacao();
        $aUrl['NfeRetAutorizacao']['method'] = self::retornaMetodoUrl($webservice->getWsretautorizacao());
        $aUrl['NfeRetAutorizacao']['version']= '3.10';

        return $aUrl;
    } // fim autoSendNFe

    private function retornaMetodoUrl($url)
    {
        $metodo = end(explode('/', $url));
        $metodo = explode('.', $metodo);

        return $metodo[0];
    }

    /**
     * autoEnv
     * Este método envias as notas fiscais passadas como parámetro e as envia para a SEFAZ
     * e em saso de sucesso no envio move o arquivo para a pasta das enviadas
     * ATENÇÃO : Existe um limite para o tamanho do arquivo a ser enviado ao SEFAZ
     *
     * @version  2.0
     * @param       $nrlote : Número do Lote
     * @param array $nfenameFiles
     * @internal param $ $
     * @return boolean TRUE sucesso ou FALSE erro
     */
    public function EnvNFe($nrlote, $nfenameFiles = array(), $metodoTransmissao)
    {
        $n = count($nfenameFiles);
        if ($n > 0) {
            //determina o numero de grupos de envio com 10 notas por grupo
            $k = ceil($n / 10);
            // as notas localizadas na pasta validadas serão enviadas em
            // grupos de até 10 notas de cada vez, sim este é um valor arbritrario eu quis até 10
            for ($i = 0; $i < $k; $i++) {
                //limpa a matriz com as notas fiscais
                $aNFE = null;
                for ($x = $i * 10; $x < (($i + 1) * 10); $x++) {
                    if ($x < $n) {
                        $filename = $this->valDir . $nfenameFiles[$x];
                        $nfefile  = file_get_contents($filename);
                        $aNFE[]   = $nfefile;
                    }
                }
                //enviar as notas
                if ($ret = $this->sendLot($aNFE, $nrlote, $metodoTransmissao)) {
                    //incrementa o numero do lote no controle
                    $nrlote++;
                    if ($this->__putNumLot($nrlote)) {
                        $this->errStatus = true;
                        $this->errMsg .= "Falha na Gravação do numero do lote de envio!!";
                        return false;
                    }
                    //mover as notas do lote para o diretorio de enviadas
                    //para cada em $aNames[] mover para $this->envDir
                    for ($x = $i * 10; $x < (($i + 1) * 10); $x++) {
                        if ($x < $n) {
                            if (!rename($this->valDir . $nfenameFiles[$x], $this->envDir . $nfenameFiles[$x])) {
                                $this->errStatus = true;
                                $this->errMsg .= " Falha na movimentação da NFe das 'validadas' para 'enviadas'!! ";
                                return false;
                            }
                        }
                    } //fim for rename
                    //incrementa o numero

                } else {
                    $this->errStatus = true;
                    $this->errMsg .= ' Erro no envio do lote de NFe!! ';
                    return false;
                }
            }
        } else {
            return false;
        }
        return true;
    }

    /**
     * Consulta Situação da Nota Fiscal na SEFAZ, caso a Situação Seja Aceita envia o Xml da
     * Nota Fiscal para o Destinatário.
     * Passar o Número da Chave OU Numero do Recibo
     * @param $recibo
     * @param $chave
     * @param $nrlotenfe
     * @param $cdempresa
     * @throws \Exception
     * @internal param $empresa
     * @internal param $protocolo
     */
    public function consultaSitNfe($recibo, $chave, $tpEmissao, $nrlotenfe, $cdempresa, $nrorg)
    {

        try {
            //Busca protocolo por recibo Ou Chave de Acesso
            $aRet = $this->getProtocol($recibo, $chave, $tpEmissao);
            //Se o Lote está processado busca os protocolos das notas enviadas
            if ($aRet['cStat'] == self::ID_LOTE_PROCESSADO) {
                self::updLoteNfe($cdempresa, $nrlotenfe, self::LOTE_PROCESSADO, $aRet['xMotivo']);
                $protocolos = $aRet['aProt'];
                if ($protocolos) {
                    foreach ($protocolos as $protocolo) {
                        switch ($protocolo['cStat']) {
                        case (self::ID_NF_AUTORIZADA):
                        case (self::ID_NF_AUTORIZADA_FORA_DO_PRAZO):
                            //NFe aprovada
                            self::alteraNotaFiscal(
                                $protocolo['chNFe'],
                                self::NF_ACEITA,
                                $protocolo['xMotivo'],
                                $protocolo['nProt'],
                                $protocolo['dhRecbto']
                            );
                            self::protocolaNota($protocolo['chNFe'], $this->aprDir);
                            self::enviaEmail($protocolo['chNFe']);
                            /** @var Notafiscal $nota */
                            $nota = self::getNotaFiscal($protocolo['chNFe']);
                            if($nota->getNrinsjurtran()){
                                self::enviaEmail($protocolo['chNFe'], 'T');
                            }
                            break;

                        case (self::ID_NF_DENEGADA):
                        case (self::ID_NF_DENEGADA_EMIT):
                        case (self::ID_NF_DENEGADA_DEST):
                            //NFe Denegada
                            self::alteraNotaFiscal(
                                $protocolo['chNFe'],
                                self::NF_DENEGADA,
                                $protocolo['xMotivo'],
                                $recibo,
                                $protocolo['dhRecbto'],
                                self::DESBLOQUADO
                            );
                            break;
                        default:
                            //Nf Rejeitada
                            self::alteraNotaFiscal(
                                $protocolo['chNFe'],
                                self::NF_REJEITADA,
                                $protocolo['xMotivo'],
                                $recibo,
                                $protocolo['dhRecbto'],
                                self::DESBLOQUADO
                            );
                        }
                    }
                }
            }
            elseif($aRet['cStat'] == self::ID_LOTE_EM_PROCESSAMENTO){
                self::updLoteNfe($cdempresa, $nrlotenfe, self::LOTE_EM_PROCESSAMENTO, $aRet['xMotivo']);
                self::alteraNotaFiscalDif104(
                    $cdempresa,
                    $nrlotenfe,
                    $nrorg,
                    'T',
                    null,
                    '',
                    $aRet['cStat'],
                    $aRet['xMotivo'],
                    self::BLOQUEADO
                );
            }
            elseif($aRet['cStat'] == self::ID_LOTE_NAO_LOCALIZADO){
                self::updLoteNfe($cdempresa, $nrlotenfe, self::LOTE_NAO_LOCALIZADO, $aRet['xMotivo']);
                self::alteraNotaFiscalDif104(
                    $cdempresa,
                    $nrlotenfe,
                    $nrorg,
                    'P',
                    null,
                    '',
                    $aRet['cStat'],
                    $aRet['xMotivo'],
                    self::DESBLOQUADO
                );
            }
            else{
                self::updLoteNfe($cdempresa, $nrlotenfe, self::PROBLEMA_RECIBO_OU_CNPJ, $aRet['xMotivo']);
                self::alteraNotaFiscalDif104(
                    $cdempresa,
                    $nrlotenfe,
                    $nrorg,
                    self::NF_REJEITADA,
                    null,
                    '',
                    $aRet['cStat'],
                    $aRet['xMotivo'],
                    self::DESBLOQUADO
                );
                $notasLote = Notafiscal::findByNrlotenfeAndCdemplotenfeAndNrorg($nrlotenfe, $cdempresa, $nrorg);
                /** @var Notafiscal $nota */
                foreach($notasLote as $nota){
                    self::moveNotaParaPasta($nota->getNracessonfe(), $this->rejDir);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * getProtocol
     * Solicita resposta do lote de Notas Fiscais ou o protocolo de
     * autorizaÃ§Ã£o da NFe
     * Caso $this->cStat == 105 Tentar novamente mais tarde
     *
     * @name               getProtocol
     * @param  string      $recibo   numero do recibo do envio do lote
     * @param  string      $chave    numero da chave da NFe de 44 digitos
     * @param   string     $tpAmb    numero do ambiente 1-producao e 2-homologaÃ§Ã£o
     * @param int|string   $modSOAP  1 usa __sendSOAP e 2 usa __sendSOAP2
     * @param array|string $aRetorno Array com os dados do protocolo
     * @throws Exception|\Exception
     * @throws \Exception
     * @return mixed    false ou xml
     */

    public function getProtocol($recibo = '', $chave = '', $tpEmissao = Notafiscal::MODO_SERIE_NORMAL, $tpAmb = '', $modSOAP = '2', &$aRetorno = '')
    {
        try {
            //carrega defaults
            $i        = 0;
            $aRetorno = array('bStat' => false, 'cStat' => '', 'xMotivo' => '', 'aProt' => '', 'aCanc' => '', 'xmlRetorno' => '');
            $cUF      = $this->cUF;
            $UF       = $this->UF;
            if ($tpAmb == '') {
                $tpAmb = $this->tpAmb;
            }
            if ($tpAmb != '1' && $tpAmb != '2') {
                $tpAmb = '2';
            }
            $aURL       = $this->aURL;
            $ctpEmissao = '';
            //verifica se a chave foi passada
            if ($chave != '') {
                //se sim extrair o cUF da chave
                $cUF        = substr($chave, 0, 2);
                $ctpEmissao = substr($chave, 34, 1);
                //testar para ver se Ã© o mesmo do emitente
                if ($cUF != $this->cUF || $tpAmb != $this->tpAmb) {
                    //se nÃ£o for o mesmo carregar a sigla
                    $UF = $this->UFList[$cUF];
                    //recarrega as url referentes aos dados passados como parametros para a funÃ§Ã£o
                    $aURL = $this->loadSEFAZ($this->raizDir . 'config' . DIRECTORY_SEPARATOR . $this->xmlURLfile, $tpAmb, $UF);
                }
            }
            //verifica se o SCAN esta habilitado
            //            if ($this->enableSCAN || $ctpEmissao == '3'){
            //                $aURL = $this->loadSEFAZ( $this->raizDir . 'config' . DIRECTORY_SEPARATOR . $this->xmlURLfile,$tpAmb,'SCAN');
            //            }
            if ($tpEmissao != Notafiscal::MODO_SERIE_NORMAL) {
                $aURL = $this->loadSEFAZ($this->raizDir . 'config' . DIRECTORY_SEPARATOR . $this->xmlURLfile, $tpAmb, $UF, true);
            }
            if ($recibo == '' && $chave == '') {
                $msg = "ERRO. Favor indicar o numero do recibo ou a chave de acesso da NFe!!";
                throw new \Exception($msg);
            }
            if ($recibo != '' && $chave != '') {
                $msg = "ERRO. Favor indicar somente um dos dois dados ou o numero do recibo ou a chave de acesso da NFe!!";
                throw new \Exception($msg);
            }
            //consulta pelo recibo
            if ($recibo != '' && $chave == '') {
                //buscar os protocolos pelo numero do recibo do lote
                //identificaÃ§Ã£o do serviÃ§o
                $servico = 'NfeRetAutorizacao';
                //recuperaÃ§Ã£o da versÃ£o
                $versao = $aURL[$servico]['version'];
                //recuperaÃ§Ã£o da url do serviÃ§o
                $urlservico = $aURL[$servico]['URL'];
                //recuperaÃ§Ã£o do mÃ©todo
                $metodo = $aURL[$servico]['method'];
                //montagem do namespace do serviÃ§o
                $namespace = $this->URLPortal . '/wsdl/' . $servico;
                //montagem do cabeÃ§alho da comunicaÃ§Ã£o SOAP
                $cabec = '<nfeCabecMsg xmlns="' . $namespace . '"><cUF>' . $cUF . '</cUF><versaoDados>' . $versao . '</versaoDados></nfeCabecMsg>';
                //montagem dos dados da mensagem SOAP
                $dados = '<nfeDadosMsg xmlns="' . $namespace . '"><consReciNFe xmlns="' . $this->URLPortal . '" versao="' . $versao . '"><tpAmb>' . $tpAmb . '</tpAmb><nRec>' . $recibo . '</nRec></consReciNFe></nfeDadosMsg>';
                //nome do arquivo
                $nomeArq = $recibo . '-protrec.xml';
            }
            //consulta pela chave
            if ($recibo == '' && $chave != '') {
                //buscar o protocolo pelo numero da chave de acesso
                //identificaÃ§Ã£o do serviÃ§o
                $servico = 'NfeConsulta';
                //recuperaÃ§Ã£o da versÃ£o
                $versao = $aURL[$servico]['version'];
                //recuperaÃ§Ã£o da url do serviÃ§o
                $urlservico = $aURL[$servico]['URL'];
                //recuperaÃ§Ã£o do mÃ©todo
                $metodo = $aURL[$servico]['method'];
                //montagem do namespace do serviÃ§o
                $namespace = $this->URLPortal . '/wsdl/' . $servico . '2';
                //montagem do cabeÃ§alho da comunicaÃ§Ã£o SOAP
                $cabec = '<nfeCabecMsg xmlns="' . $namespace . '"><cUF>' . $cUF . '</cUF><versaoDados>' . $versao . '</versaoDados></nfeCabecMsg>';
                //montagem dos dados da mensagem SOAP
                $dados = '<nfeDadosMsg xmlns="' . $namespace . '"><consSitNFe xmlns="' . $this->URLPortal . '" versao="' . $versao . '"><tpAmb>' . $tpAmb . '</tpAmb><xServ>CONSULTAR</xServ><chNFe>' . $chave . '</chNFe></consSitNFe></nfeDadosMsg>';
            }
            //envia a solicitaÃ§Ã£o via SOAP
            if ($modSOAP == 2) {
                $retorno = $this->__sendSOAP2($urlservico, $namespace, $cabec, $dados, $metodo, $tpAmb);
            } else {
                $retorno = $this->__sendSOAP($urlservico, $namespace, $cabec, $dados, $metodo, $tpAmb, $UF);
            }
            //verifica o retorno
            if ($retorno) {
                //tratar dados de retorno
                $doc                     = new \DOMDocument('1.0', 'utf-8'); //cria objeto DOM
                $doc->formatOutput       = false;
                $doc->preserveWhiteSpace = false;
                $doc->loadXML($retorno, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
                $cStat = !empty($doc->getElementsByTagName('cStat')->item(0)->nodeValue) ? $doc->getElementsByTagName('cStat')->item(0)->nodeValue : '';
                if ($cStat == '') {
                    //houve erro
                    $msg = "Erro cStat estÃ¡ vazio.";
                    throw new \Exception($msg);
                }
                $envelopeBodyNode = $doc->getElementsByTagNameNS('http://www.w3.org/2003/05/soap-envelope', 'Body')->item(0)->childNodes->item(0);
                //Disponibiliza o conteÃºdo xml do pacote de resposta (soap:Body) atravÃ©s do array de retorno
                $aRetorno['xmlRetorno'] = $doc->saveXML($envelopeBodyNode);
                //o retorno vai variar se for buscado o protocolo ou recibo
                //Retorno nda consulta pela Chave da NFe
                //retConsSitNFe 100 aceita 110 denegada 101 cancelada ou outro recusada
                // cStat xMotivo cUF chNFe protNFe retCancNFe
                if ($chave != '') {
                    $aRetorno['bStat']   = true;
                    $aRetorno['cStat']   = $doc->getElementsByTagName('cStat')->item(0)->nodeValue;
                    $aRetorno['xMotivo'] = !empty($doc->getElementsByTagName('xMotivo')->item(0)->nodeValue) ? $doc->getElementsByTagName('xMotivo')->item(0)->nodeValue : '';
                    $infProt             = $doc->getElementsByTagName('infProt')->item(0);
                    $infCanc             = $doc->getElementsByTagName('infCanc')->item(0);
                    $procEventoNFe       = $doc->getElementsByTagName('procEventoNFe');
                    $aProt               = '';
                    if (isset($infProt)) {
                        foreach ($infProt->childNodes as $t) {
                            $aProt[$t->nodeName] = $t->nodeValue;
                        }
                        $aProt['dhRecbto'] = !empty($aProt['dhRecbto']) ? date("d/m/Y H:i:s", $this->__convertTime($aProt['dhRecbto'])) : '';
                    }
                    $aCanc = '';
                    if (isset($infCanc)) {
                        foreach ($infCanc->childNodes as $t) {
                            $aCanc[$t->nodeName] = $t->nodeValue;
                        }
                        $aCanc['dhRecbto'] = !empty($aCanc['dhRecbto']) ? date("d/m/Y H:i:s", $this->__convertTime($aCanc['dhRecbto'])) : '';
                    }
                    $aEventos = '';
                    if (!empty($procEventoNFe)) {
                        foreach ($procEventoNFe as $i => $evento) {
                            $infEvento = $evento->getElementsByTagName('infEvento')->item(0);
                            foreach ($infEvento->childNodes as $t) {
                                if ('detEvento' == $t->nodeName) {
                                    foreach ($t->childNodes as $t2) {
                                        $aEventos[$i][$t->nodeName][$t2->nodeName] = $t2->nodeValue;
                                    }
                                    continue;
                                }
                                $aEventos[$i][$t->nodeName] = $t->nodeValue;
                            }
                            $aEventos[$i]['id'] = $infEvento->getAttribute('Id');
                        }
                    }
                    $aRetorno['aProt']    = $aProt;
                    $aRetorno['aCanc']    = $aCanc;
                    $aRetorno['aEventos'] = $aEventos;
                    //gravar o retorno na pasta temp apenas se a nota foi aprovada ou denegada
                    if ($aRetorno['cStat'] == 100 || $aRetorno['cStat'] == 101 || $aRetorno['cStat'] == 110 || $aRetorno['cStat'] == 301 || $aRetorno['cStat'] == 302) {
                        //nome do arquivo
                        $nomeArq = $chave . '-prot.xml';
                        $nome    = $this->temDir . $nomeArq;
                        $nome    = $doc->save($nome);
                    }
                }
                //Retorno da consulta pelo recibo
                //NFeRetRecepcao 104 tem retornos
                //nRec cStat xMotivo cUF cMsg xMsg protNfe* infProt chNFe dhRecbto nProt cStat xMotivo
                if ($recibo != '') {
                    $aRetorno['bStat'] = true;
                    // status do serviÃ§o
                    $aRetorno['cStat'] = $doc->getElementsByTagName('cStat')->item(0)->nodeValue;
                    // motivo da resposta (opcional)
                    $aRetorno['xMotivo'] = !empty($doc->getElementsByTagName('xMotivo')->item(0)->nodeValue) ? $doc->getElementsByTagName('xMotivo')->item(0)->nodeValue : '';
                    // numero do recibo consultado
                    $aRetorno['nRec'] = !empty($doc->getElementsByTagName('nRec')->item(0)->nodeValue) ? $doc->getElementsByTagName('nRec')->item(0)->nodeValue : '';
                    // tipo de ambiente
                    $aRetorno['tpAmb'] = !empty($doc->getElementsByTagName('tpAmb')->item(0)->nodeValue) ? $doc->getElementsByTagName('tpAmb')->item(0)->nodeValue : '';
                    // versao do aplicativo que recebeu a consulta
                    $aRetorno['verAplic'] = !empty($doc->getElementsByTagName('verAplic')->item(0)->nodeValue) ? $doc->getElementsByTagName('verAplic')->item(0)->nodeValue : '';
                    // codigo da UF que atendeu a solicitacao
                    $aRetorno['cUF'] = !empty($doc->getElementsByTagName('cUF')->item(0)->nodeValue) ? $doc->getElementsByTagName('cUF')->item(0)->nodeValue : '';
                    // codigo da mensagem da SEFAZ para o emissor (opcional)
                    $aRetorno['cMsg'] = !empty($doc->getElementsByTagName('cMsg')->item(0)->nodeValue) ? $doc->getElementsByTagName('cMsg')->item(0)->nodeValue : '';
                    // texto da mensagem da SEFAZ para o emissor (opcional)
                    $aRetorno['xMsg'] = !empty($doc->getElementsByTagName('xMsg')->item(0)->nodeValue) ? $doc->getElementsByTagName('xMsg')->item(0)->nodeValue : '';
                    if ($cStat == '104') {
                        //aqui podem ter varios retornos dependendo do numero de NFe enviadas no Lote e jÃ¡ processadas
                        $protNfe = $doc->getElementsByTagName('protNFe');
                        foreach ($protNfe as $d) {
                            $infProt   = $d->getElementsByTagName('infProt')->item(0);
                            $protcStat = $infProt->getElementsByTagName('cStat')->item(0)->nodeValue;//cStat
                            //pegar os dados do protolo para retornar
                            foreach ($infProt->childNodes as $t) {
                                $aProt[$i][$t->nodeName] = $t->nodeValue;
                            }
                            $i++; //incluido increment para controlador de indice do array
                            //salvar o protocolo somente se a nota estiver approvada ou denegada
                            if ($protcStat == 100 || $protcStat == 110 || $protcStat == 301 || $protcStat == 302) {
                                $nomeprot = $this->temDir . $infProt->getElementsByTagName('chNFe')->item(0)->nodeValue . '-prot.xml';//id da nfe
                                //salvar o protocolo em arquivo
                                $novoprot                     = new \DOMDocument('1.0', 'UTF-8');
                                $novoprot->formatOutput       = true;
                                $novoprot->preserveWhiteSpace = false;
                                $pNFe                         = $novoprot->createElement("protNFe");
                                $pNFe->setAttribute("versao", "3.10");
                                // Importa o node e todo o seu conteudo
                                $node = $novoprot->importNode($infProt, true);
                                // acrescenta ao node principal
                                $pNFe->appendChild($node);
                                $novoprot->appendChild($pNFe);
                                $xml = $novoprot->saveXML();
                                $xml = str_replace('<?xml version="1.0" encoding="UTF-8  standalone="no"?>', '<?xml version="1.0" encoding="UTF-8"?>', $xml);
                                $xml = str_replace(array("default:", ":default"), "", $xml);
                                $xml = str_replace("\n", "", $xml);
                                $xml = str_replace("  ", " ", $xml);
                                $xml = str_replace("  ", " ", $xml);
                                $xml = str_replace("  ", " ", $xml);
                                $xml = str_replace("  ", " ", $xml);
                                $xml = str_replace("  ", " ", $xml);
                                $xml = str_replace("> <", "><", $xml);
                                file_put_contents($nomeprot, $xml);
                            } //fim protcSat
                        } //fim foreach
                    }//fim cStat
                    //converter o horÃ¡rio do recebimento retornado pela SEFAZ em formato padrÃ£o
                    if (isset($aProt)) {
                        foreach ($aProt as &$p) {
                            $p['dhRecbto'] = !empty($p['dhRecbto']) ? date("d/m/Y H:i:s", $this->__convertTime($p['dhRecbto'])) : '';
                        }
                    } else {
                        $aProt = array();
                    }
                    $aRetorno['aProt'] = $aProt; //passa o valor de $aProt para o array de retorno
                    $nomeArq           = $recibo . '-recprot.xml';
                    $nome              = $this->temDir . $nomeArq;
                    $nome              = $doc->save($nome);
                } //fim recibo
            } else {
                $msg = "Nao houve retorno Soap verifique a mensagem de erro e o debug!!";
                throw new \Exception($msg);
            } //fim retorno
        } catch (\Exception $e) {
            throw $e;
            return false;
        }//fim catch
        return $aRetorno; //mudar para $retorno
    }

    private function updLoteNfe($cdempresa, $nrlote, $status, $situacao)
    {
        /** @var $lotenfe Lotenfe */
        $lotenfe = Lotenfe::findOneByCdemplotenfeAndNrlotenfe($cdempresa, $nrlote);
        $lotenfe->setIdstatuslotenfe($status);
        $lotenfe->setDssitlotenfe($situacao);
        $lotenfe->persist();
        Registry::getEntityManager()->flush();
    }

    private function alteraNotaFiscal($chaveDeAcesso, $idstatusnfe, $dsobsstatusnfe, $nrprotocolonfe, $dthrprotocolnfe, $idbloq = null)
    {

        try {
            /** @var $notafiscal NotaFiscal */
            $notafiscal = self::getNotaFiscal($chaveDeAcesso);
            if ($notafiscal) {
                $notafiscal->setIdstatusnfe($idstatusnfe);
                $notafiscal->setDsobsstatusnfe($dsobsstatusnfe);
                $notafiscal->setNrprotocolonfe($nrprotocolonfe);
                $dthrprotocolnfe = self::formatData($dthrprotocolnfe);
                $notafiscal->setDthrprotocolnfe($dthrprotocolnfe);
                if ($idbloq) {
                    $notafiscal->setIdbloqueado($idbloq);
                }
                $notafiscal->persist();

                /** @var  $relacnflotenfe Relacnflotenfe */
                $relacnflotenfe = Relacnflotenfe::findOneByCdfilialAndNrlanctonfAndNrorgAndNrlotenfe($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg(), $notafiscal->getNrlotenfe());
                $relacnflotenfe->setIdstatusrelnfe($idstatusnfe);
                $relacnflotenfe->persist();

                Registry::getEntityManager()->flush();
            } else {
                throw new \Exception("Nota Fiscal não Encontrada pela chave de acesso: {$chaveDeAcesso}.");
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function alteraNotaFiscalDif104($cdempresa, $nrlotenfe, $nrorg, $idstatusnfe, $dataProtocolo, $nrProtocolo, $cdstatus, $dsstatus, $idbloq = null)
    {

        try {
            $notafiscal = Notafiscal::findOneByCdemplotenfeAndNrlotenfeAndNrorg($cdempresa, $nrlotenfe, $nrorg);
            if ($notafiscal instanceof Notafiscal) {
                $notafiscal->setIdstatusnfe($idstatusnfe);
                $notafiscal->setDsobsstatusnfe($dsstatus);
                if ($idbloq) {
                    $notafiscal->setIdbloqueado($idbloq);
                }
                $notafiscal->persist();

                /** @var  $relacnflotenfe Relacnflotenfe */
                $relacnflotenfe = Relacnflotenfe::findOneByCdemplotenfeAndNrlotenfeAndNrorg($cdempresa, $nrlotenfe, $nrorg);
                $relacnflotenfe->setDtprotocolrelnfe($dataProtocolo);
                $relacnflotenfe->setNrprotocolrelnfe($nrProtocolo);
                $relacnflotenfe->setIdstatusrelnfe($idstatusnfe);
                $relacnflotenfe->setCdstatusrelnfe($cdstatus);
                $relacnflotenfe->setDsstatusrelnfe($dsstatus);
                $relacnflotenfe->persist();

                Registry::getEntityManager()->flush();
            } else {
                throw new \Exception("Nota fiscal não encontrada");
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $chaveDeAcesso
     * @return $notafiscal
     */
    private function getNotaFiscal($chaveDeAcesso)
    {
        /**@var  NotaFiscal */
        $notafiscal = Notafiscal::findOneByNracessonfe($chaveDeAcesso);
        return $notafiscal;
    }

    private function formatData($data)
    {
        try {
            if ($data instanceof \Setforms2\Shared\ORM\DateTime === false) {
                $data = DateUtil::getDataDeString($data, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            }
            return $data;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function protocolaNota($chaveAcesso, $pasta)
    {
        $protFile = $this->temDir . $chaveAcesso . '-prot.xml';
        $arq      = self::buscaArquivoNfe($chaveAcesso, $this->envDir);
        if ($arq) {
            $procnfe                  = $this->addProt($arq['File'], $protFile);
            $prot                     = new \DOMDocument(); //cria objeto DOM
            $prot->formatOutput       = false;
            $prot->preserveWhiteSpace = false;
            $prot->loadXML($procnfe);
            //salvar a NFe com o protocolo na pasta
            if ($prot->save($pasta . $arq['NameFile'])) {
                //se o arquivo foi gravado na pasta destino com sucesso
                //remover os arquivos das outras pastas
                unlink($arq['File']);
                unlink($protFile);
            } //endif
        } else {
            throw new \Exception("Nao foi encontrada Nota Fiscal enviada relacionada a Chave de Acesso: {$chaveAcesso}");
        }
    }

    private function moveNotaParaPasta($chaveAcesso, $pasta){
        $arq = self::buscaArquivoNfe($chaveAcesso, $this->envDir);
        copy($arq['File'], $pasta . $arq['NameFile']);
        unlink($arq['File']);
    }

    /**
     * Busca na pasta passada como parâmetro um arquivo -Nfe e o Nome do Arquivo de Acordo com A Chave de Acesso. Caso não encontre retorna false;
     * @param $chaveAcesso
     * @param $dir
     * @return bool|array
     */
    public function buscaArquivoNfe($chaveAcesso, $dir, $tipo = null)
    {
        if($tipo == 'C' || $tipo == 'CT'){
            $aName = $this->listDir($dir, '*procCanc.xml', false);
        }
        else{
            $aName = $this->listDir($dir, '*-nfe.xml', false);
        }
        //se houver algum arquivo *-nfe.xml continua, caso contrario sai
        if (count($aName) > 0) {
            foreach ($aName as $file) {
                $nfeFile = $dir . $file;
                $idNFe   = substr($file, 0, 44);
                if (is_file($nfeFile)) {
                    $xml                    = file_get_contents($nfeFile);
                    $doc                    = new \DOMDocument();
                    $doc->preservWhiteSpace = false; //elimina espaÃ§os em branco
                    $doc->formatOutput      = false;
                    $doc->loadXML($xml);
                    if($tipo == 'C' || $tipo == 'CT'){
                        $node = $doc->getElementsByTagName('chNFe')->item(0);
                        //obtem chave de acesso do xml
                        $chaveFileNfe = trim($node->nodeValue);
                    }
                    else{
                        $node = $doc->getElementsByTagName('infNFe')->item(0);
                        //obtem chave de acesso do xml
                        $chaveFileNfe = substr(trim($node->getAttribute("Id")), 3);
                    }
                    if ($chaveFileNfe === $chaveAcesso) {
                        return array('File'     => $nfeFile,
                                    'NameFile' => $idNFe);
//                        return $nfeFile;
                    }
                }
            }
        }
        return false;
    }

    public function enviaEmail($chaveAcesso, $tipo = null)
    {
        try {
            /** @var Notafiscal $notafiscal */
            $notafiscal = self::getNotaFiscal($chaveAcesso);
            $de         = self::getEmailRemetente($notafiscal);
            $para       = self::getEmailDestinatario($notafiscal, $tipo);
            if ($de and $para) {
                $assunto         = self::getAssunto($chaveAcesso, $tipo);
                $mensagem        = self::getMensagem($notafiscal, $tipo);
                $envio           = Notifier::factory();
                $destinatarios   = array();
                $destinatarios[] = self::formatDestinatatios($para);
                $anexo           = array();
                $anexo           = self::buscaAnexos($chaveAcesso, $tipo);
                $notification    = new Notification();
                $notification->setSubject($assunto);
                $notification->setAttachments($anexo);
                $notification->setFrom($de);
                $notification->setTo($destinatarios);
                $notification->setBody($mensagem);
                $envio->setNotificationBusiness($notification);
                $envio->send();

                if($tipo != 'C' && $tipo != 'CT'){
                    unlink($this->config["caminhoPDFreports"] . $chaveAcesso . '.pdf');
                }

                if($tipo == 'T' || $tipo == 'CT'){
                    $notafiscal->setIdemailnfetransp('E');
                }
                else{
                    $notafiscal->setIdemailnfe('E');
                }
                Registry::getEntityManager()->flush();
            }
        } catch (\Exception $e) {
            tk_message("Ocorreu erro durante envio do Email: {$e}");
        }
    }

    /**
     * Busca Endereço de Email cadastrado na Filial
     * *
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     * @internal param $chaveAcesso
     * @return mixed
     */
    private function getEmailRemetente(Notafiscal $notafiscal)
    {

        /** @var $endefili Endefili */
        $endefili = Endefili::findOneByCdfilialAndIdtpendefiliAndNrorg(
            $notafiscal->getCdfiliemit(),
            self::TIPO_END_PRINCIPAL,
            $notafiscal->getNrorg()
        );
        return $endefili->getDsemailemxmlnfe() == true ? $endefili->getDsemailemxmlnfe() : $endefili->getDsemailfili();
    }

    private function getEmailDestinatario(Notafiscal $notafiscal, $tipo)
    {
        $dadosEmail = Notafiscal::getRepository()->getDadosEmalNfe($notafiscal->getCdfiliemit(), $notafiscal->getNrlanctonf());

        if(!empty($dadosEmail)){
            if($tipo == 'T' || $tipo == 'CT'){
                return $dadosEmail[0]['DSEMAILTRANSPNFE'];
            }
            return $dadosEmail[0]['DSEMAILDEST'];
        }
    }

    private function getAssunto($chaveacesso, $tipo)
    {
        if($tipo == 'C' || $tipo == 'CT'){
            return "Cancelamento de Nota Fiscal Eletrônica. Chave: {$chaveacesso}";
        }
        return "Emissão de Nota Fiscal Eletrônica. Chave: {$chaveacesso}";
    }

    private function getMensagem(Notafiscal $notafiscal, $tipo)
    {

        $dadosEmail = Notafiscal::getRepository()->getDadosEmalNfe(
            $notafiscal->getCdfiliemit(),
            $notafiscal->getNrlanctonf()
        );
        $dadosEmail = $dadosEmail[0];

        if($tipo == 'C' || $tipo == 'CT'){
            $mensagem   = " E-mail referente ao cancelamento da Nota Fiscal Eletrônica nº: {$dadosEmail['NRNOTAFISCALELE']}
                      <br /><br />Chave de Acesso: {$dadosEmail['NRACESSONFE']}
                      <br /><br />Emitente: {$dadosEmail['NMRAZSOCORIG']}
                      <br />Destinatário: {$dadosEmail['NMRAZSOCDEST']}
                      <br /><br />{$dadosEmail['DSMENSEMAILNFE']} ";
        }
        else{
            $mensagem   = " E-mail referente a emissão da Nota Fiscal Eletrônica nº: {$dadosEmail['NRNOTAFISCALELE']}
                      <br /><br />Chave de Acesso: {$dadosEmail['NRACESSONFE']}
                      <br /><br />Emitente: {$dadosEmail['NMRAZSOCORIG']}
                      <br />Destinatário: {$dadosEmail['NMRAZSOCDEST']}
                      <br /><br />{$dadosEmail['DSMENSEMAILNFE']} ";
        }

        return $mensagem;
    }

    private function formatDestinatatios($emails)
    {
        if (strripos($emails, ',') > 0) {
            str_replace(" ", "", $emails);
            $emails = explode(',', $emails);
        }
        return $emails;
    } //fim autoValidNFe

    /**
     * Busca arquivo xml e Danfe da Nota Fiscal
     * @param $chaveAcesso
     * @return array
     */
    private function buscaAnexos($chaveAcesso, $tipo)
    {
        $diretorio    = ($tipo == 'C' || $tipo == 'CT') ? $this->canDir : $this->aprDir;
        $arquivoxml   = self::buscaArquivoNfe($chaveAcesso, $diretorio, $tipo);
        //$arquivoxml   = $diretorio . $arquivoxml['NameFile'];

        if($tipo == 'C' || $tipo == 'CT'){
            $arquivos     = array($arquivoxml['File']);
            return $arquivos;
        }

        $arquivodanfe = self::getDanfe($chaveAcesso);
        $arquivos     = array($arquivoxml['File'], $arquivodanfe);
        return $arquivos;
    }//fim envCCe

    /**
     * loadSEFAZ
     * Função para extrair o URL, nome do serviço e versão dos webservices das SEFAZ de
     * todos os Estados da Federação do arquivo urlWebServicesNFe.xml
     *
     * O arquivo xml é estruturado da seguinte forma :
     * <WS>
     *   <UF>
     *      <sigla>AC</sigla>
     *          <homologacao>
     *              <Recepcao service='nfeRecepcao' versao='1.10'>http:// .....
     *              ....
     *          </homologacao>
     *          <producao>
     *              <Recepcao service='nfeRecepcao' versao='1.10'>http:// ....
     *              ....
     *          </producao>
     *   </UF>
     *   <UF>
     *      ....
     * </WS>
     *
     * @name          loadSEFAZ
     * @version 1.1
     * @package NFePHP
     * @param  string $spathXML Caminho completo para o arquivo xml
     * @param  string $tpAmb    Pode ser "2-homologacao" ou "1-producao"
     * @param  string $sUF      Sigla da Unidade da Federação (ex. SP, RS, etc..)
     * @return mixed             FALSE se houve erro ou array com os dado do URLs das SEFAZ
     */
    //Retirado o primeiro parametro $spathXML. Urls do webservice agora são armazenadas no banco.
    private function getDanfe($chaveacesso)
    {
        try {
            /** @var $notafiscal Notafiscal */
            $notafiscal = self::getNotaFiscal($chaveacesso);
            $modelorel  = Facade::getModeloDanfe($notafiscal);
            $params     = array(
                "FILIAL"     => $notafiscal->getCdfiliemit(),
                "LANCAMENTO" => $notafiscal->getNrlanctonf(),
            );
            tk_generate_qr2_report_from_array($modelorel, $params, $chaveacesso . '.pdf');
            copy($this->config["caminhoPDFreports"] . $chaveacesso . '.pdf', $this->pdfDir . $chaveacesso . '.pdf');
            return $this->config["caminhoPDFreports"] . $chaveacesso . '.pdf';
        } catch (\Exception $e) {
            tk_message("Ocorreu erro durante geração do Danfe: {$e}");
        }
    } //fim loadSEFAZ

    /** Validação de um Xml com Xsds
     * @param $filename
     * @throws \Exception
     * @internal param filename nome do aqruivo XML a ser validado
     * @return bool
     */
    public function validNFe($filename)
    {
        try {
            if ($nfefile = file_get_contents($this->assDir . $filename)) {
                //validar
                //como os arquivos xsd são atualizados e tem sua verção alterada
                //devemos burcar este arquivo da versão correta
                //para isso temos que obter o numero da versão na própria nfe
                $xmldoc                    = new \DOMDocument();
                $xmldoc->preservWhiteSpace = false; //elimina espaços em branco
                $xmldoc->formatOutput      = false;
                $xmldoc->loadXML($nfefile, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
                //buscar o nome do scheme
                $filexsd = $this->listDir($this->xsdDir . $this->schemeVer . DIRECTORY_SEPARATOR, 'nfe_v*.xsd', true);
                if (!$filexsd) {
                    $this->errMsg    = 'Erro na localização do schema xsd';
                    $this->errStatus = true;
                    return false;
                }
                if ($this->validXML($nfefile, $filexsd[0])) {
                    // validado => transferir para pasta validados
                    $file = $this->valDir . $filename;
                    if (!file_put_contents($file, $nfefile)) {
                        $this->errStatus = true;
                    } else {
                        unlink($nfefile = $this->assDir . $filename);
                    }
                } else {
                    //NFe com erros transferir de pasta rejeitadas
                    $file = $this->rejDir . $filename;
                    if (!file_put_contents($file, $nfefile)) {
                        $this->errStatus = true;
                    } else {
                        unlink($nfefile = $this->assDir . $filename);
                    } //fim teste de gravação
                } //fim validação
            } //fim teste de leitura

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * envCCe
     * Envia carta de correção da Nota Fiscal para a SEFAZ.
     *
     * @param   string   $chNFe      Chave da NFe
     * @param   string   $xCorrecao  Descrição da Correção entre 15 e 1000 caracteres
     * @param   string   $nSeqEvento numero sequencial da correção d 1 até 20
     *                               isso deve ser mantido na base de dados e
     *                               as correções consolidadas, isto é a cada nova correção
     *                               devem ser inclusas as anteriores no texto.
     *                               O Web Service não permite a duplicidade de numeração
     *                               e nem controla a ordem crescente
     * @param int|string $tpAmb      Tipo de ambiente
     * @param int|string $modSOAP    1 usa __sendSOP e 2 usa __sendSOAP2
     * @param            $numLote
     * @internal param $envCCe
     * @version  0.1.5
     * @package  NFePHP
     * @author   Roberto L. Machado <linux.rlm at gmail dot com>
     * @return    mixed false ou xml com a CCe
     */
    public function envCCe($chNFe = '', $xCorrecao = '', $nSeqEvento = '1', $tpAmb = '', $modSOAP = '2', $numLote)
    {
        //testa se os dados da carta de correção foram passados
        if ($chNFe == '' || $xCorrecao == '') {
            $this->errStatus = true;
            $this->errMsg    = "Dados para a carta de correção não podem ser vazios";
            return false;
        }
        if (strlen($chNFe) != 44) {
            $this->errStatus = true;
            $this->errMsg    = "Uma chave de NFe válida não foi passada como parâmetro.";
            return false;
        }
        //se o numero sequencial do evento não foi informado ou se for maior que 1 digito
        if ($nSeqEvento == '' || strlen($nSeqEvento) > 2 || !is_numeric($nSeqEvento)) {
            $this->errStatus = true;
            $this->errMsg .= "Número sequencial da correção não encontrado ou é maior que 99 ou contêm caracteres não numéricos [$nSeqEvento]";
            return false;
        }
        if (strlen($xCorrecao) < 15 || strlen($xCorrecao) > 1000) {
            $this->errStatus = true;
            $this->errMsg .= "O texto da correção deve ter entre 15 e 1000 caracteres!";
            return false;
        }
        //limpa o texto de correção para evitar surpresas
        $xCorrecao = $this->__cleanString($xCorrecao);
        //ajusta ambiente
        if ($tpAmb == '') {
            $tpAmb = $this->tpAmb;
        }
        //decompor a chNFe e pegar o tipo de emissão
        $tpEmiss = substr($chNFe, 34, 1);
        //verifica se o SCAN esta habilitado
        if (!$this->enableSCAN) {
            $aURL = $this->aURL;
        } else {
            $aURL = $this->loadSEFAZ($this->raizDir . 'config' . DIRECTORY_SEPARATOR . $this->xmlURLfile, $tpAmb, $this->config["UF"], true);
        }
        //Data e hora do evento no formato AAAA-MM-DDTHH:MM:SSTZD (UTC)
        $dhEvento = date('Y-m-d') . 'T' . date('H:i:s') . $this->timeZone;
        //se o envio for para svan mudar o numero no orgão para 90
//        if ($this->enableSVAN) {
//            $cOrgao = '90';
//        } else {
            $cOrgao = $this->cUF;
//        }
        //montagem do namespace do serviço
        $servico = 'RecepcaoEvento';
        //recuperação da versão
        $versao = $aURL[$servico]['version'];
        //recuperação da url do serviço
        $urlservico = $aURL[$servico]['URL'];
        //recuperação do método
        $metodo = $aURL[$servico]['method'];
        //montagem do namespace do serviço
        $namespace = $this->URLPortal . '/wsdl/' . $servico;

        $tpEvento = '110110';
        //de acordo com o manual versão 5 de março de 2012
        // 2   +    6     +    44         +   2  = 54 digitos
        //“ID” + tpEvento + chave da NF-e + nSeqEvento
        //garantir que existam 2 digitos em nSeqEvento para montar o ID com 54 digitos
        if (strlen(trim($nSeqEvento)) == 1) {
            $zenSeqEvento = str_pad($nSeqEvento, 2, "0", STR_PAD_LEFT);
        } else {
            $zenSeqEvento = trim($nSeqEvento);
        }

        $id         = "ID" . $tpEvento . $chNFe . $zenSeqEvento;
        $descEvento = 'Carta de Correcao';
        $xCondUso   = 'A Carta de Correcao e disciplinada pelo paragrafo 1o-A do art. 7o do Convenio S/N, de 15 de dezembro de 1970 e pode ser utilizada para regularizacao de erro ocorrido na emissao de documento fiscal, desde que o erro nao esteja relacionado com: I - as variaveis que determinam o valor do imposto tais como: base de calculo, aliquota, diferenca de preco, quantidade, valor da operacao ou da prestacao; II - a correcao de dados cadastrais que implique mudanca do remetente ou do destinatario; III - a data de emissao ou de saida.';

        $Ev = '';
        $Ev .= "<evento xmlns=\"$this->URLPortal\" versao=\"$versao\">";
        $Ev .= "<infEvento Id=\"$id\">";
        $Ev .= "<cOrgao>$cOrgao</cOrgao>";
        $Ev .= "<tpAmb>$tpAmb</tpAmb>";
        $Ev .= "<CNPJ>$this->cnpj</CNPJ>";
        $Ev .= "<chNFe>$chNFe</chNFe>";
        $Ev .= "<dhEvento>$dhEvento</dhEvento>";
        $Ev .= "<tpEvento>$tpEvento</tpEvento>";
        $Ev .= "<nSeqEvento>".ltrim($nSeqEvento, 0)."</nSeqEvento>";
        $Ev .= "<verEvento>$versao</verEvento>";
        $Ev .= "<detEvento versao=\"$versao\">";
        $Ev .= "<descEvento>$descEvento</descEvento>";
        $Ev .= "<xCorrecao>$xCorrecao</xCorrecao>";
        $Ev .= "<xCondUso>$xCondUso</xCondUso>";
        $Ev .= "</detEvento></infEvento></evento>";
        //assinatura dos dados
        $tagid = 'infEvento';
        $Ev    = $this->signXML($Ev, $tagid);
        $Ev    = str_replace('<?xml version="1.0"?>', '', $Ev);
        $Ev    = str_replace('<?xml version="1.0" encoding="utf-8"?>', '', $Ev);
        $Ev    = str_replace('<?xml version="1.0" encoding="UTF-8"?>', '', $Ev);
        $Ev    = str_replace(array("\r", "\n", "\\s"), "", $Ev);
        //carrega uma matriz temporária com os eventos assinados
        //montagem dos dados
        $dados = '';
        $dados .= "<envEvento xmlns=\"$this->URLPortal\" versao=\"$versao\">";
        $dados .= "<idLote>$numLote</idLote>";
        $dados .= $Ev;
        $dados .= "</envEvento>";
        //montagem da mensagem
        $cabec = "<nfeCabecMsg xmlns=\"$namespace\"><cUF>$this->cUF</cUF><versaoDados>$versao</versaoDados></nfeCabecMsg>";
        $dados = "<nfeDadosMsg xmlns=\"$namespace\">$dados</nfeDadosMsg>";
        //grava solicitação em temp
        if (!file_put_contents($this->temDir . "$chNFe-$nSeqEvento-envCCe.xml", $Ev)) {
            $this->errStatus = true;
            $this->errMsg    = "Falha na gravação da CCe!!\n";
        }

        //envia dados via SOAP
        if ($modSOAP == '2') {
            $retorno = $this->__sendSOAP2($urlservico, $namespace, $cabec, $dados, $metodo, $tpAmb);
        } else {
            $retorno = $this->__sendSOAP($urlservico, $namespace, $cabec, $dados, $metodo, $tpAmb, $this->UF);
        }
        //verifica o retorno
        if (!$retorno) {
            //não houve retorno
            $this->errStatus = true;
            $this->errMsg    = "Nao houve retorno Soap verifique a mensagem de erro e o debug!!\n";
            return false;
        }
        //tratar dados de retorno
        $xmlretCCe                     = new \DOMDocument('1.0', 'utf-8'); //cria objeto DOM
        $xmlretCCe->formatOutput       = false;
        $xmlretCCe->preserveWhiteSpace = false;
        $xmlretCCe->loadXML($retorno, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
        $retEvento = $xmlretCCe->getElementsByTagName("retEvento")->item(0);
        $cStat     = !empty($retEvento->getElementsByTagName('cStat')->item(0)->nodeValue) ? $retEvento->getElementsByTagName('cStat')->item(0)->nodeValue : '';
        $xMotivo   = !empty($retEvento->getElementsByTagName('xMotivo')->item(0)->nodeValue) ? $retEvento->getElementsByTagName('xMotivo')->item(0)->nodeValue : '';
        if ($cStat == '') {
            //houve erro
            $this->errStatus = true;
            $this->errMsg    = "cStat está em branco, houve erro na comunicação Soap verifique a mensagem de erro e o debug!!\n";
            return false;
        }
        //erro no processamento cStat <> 128
        if ($cStat != 135) {
            //se cStat <> 135 houve erro e o lote foi rejeitado
            $this->errStatus = true;
            $this->errMsg    = "$cStat - $xMotivo\n";
            return false;
        }
        //a correção foi aceita cStat == 135
        //carregar a CCe
        $xmlenvCCe                     = new \DOMDocument('1.0', 'utf-8'); //cria objeto DOM
        $xmlenvCCe->formatOutput       = false;
        $xmlenvCCe->preserveWhiteSpace = false;
        $xmlenvCCe->loadXML($Ev, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
        $evento = $xmlenvCCe->getElementsByTagName("evento")->item(0);

        //Processo completo solicitação + protocolo
        $xmlprocCCe = new \DOMDocument('1.0', 'utf-8');; //cria objeto DOM
        $xmlprocCCe->formatOutput       = false;
        $xmlprocCCe->preserveWhiteSpace = false;
        //cria a tag procEventoNFe
        $procEventoNFe = $xmlprocCCe->createElement('procEventoNFe');
        $xmlprocCCe->appendChild($procEventoNFe);
        //estabele o atributo de versão
        $eventProc_att1 = $procEventoNFe->appendChild($xmlprocCCe->createAttribute('versao'));
        $eventProc_att1->appendChild($xmlprocCCe->createTextNode($versao));
        //estabelece o atributo xmlns
        $eventProc_att2 = $procEventoNFe->appendChild($xmlprocCCe->createAttribute('xmlns'));
        $eventProc_att2->appendChild($xmlprocCCe->createTextNode($this->URLPortal));
        //carrega o node evento
        $node1 = $xmlprocCCe->importNode($evento, true);
        $procEventoNFe->appendChild($node1);
        //carrega o node retEvento
        $node2 = $xmlprocCCe->importNode($retEvento, true);
        $procEventoNFe->appendChild($node2);
        //salva o xml como string em uma variável
        $procXML = $xmlprocCCe->saveXML();
        //remove as informações indesejadas
        $procXML = str_replace("xmlns:default=\"http://www.w3.org/2000/09/xmldsig#\"", '', $procXML);
        $procXML = str_replace('default:', '', $procXML);
        $procXML = str_replace(':default', '', $procXML);
        $procXML = str_replace("\n", '', $procXML);
        $procXML = str_replace("\r", '', $procXML);
        $procXML = str_replace("\\s", '', $procXML);
        //salva o arquivo xml
        if (!file_put_contents($this->cccDir . "$chNFe-$nSeqEvento-procCCe.xml", $procXML)) {
            $this->errStatus = true;
            $this->errMsg    = "Falha na gravação da procCCe!!\n";
        }
        return $procXML;
    } //fim statusServico

    /**
     * statusServico
     * Verifica o status do servico da SEFAZ
     *
     * $this->cStat = 107 OK
     *        cStat = 108 sitema paralizado momentaneamente, aguardar retorno
     *        cStat = 109 sistema parado sem previsao de retorno, verificar status SCAN
     * se SCAN estiver ativado usar, caso caontrario aguardar pacientemente.
     * @name             statusServico
     * @version  2.0
     * @package  NFePHP
     * @param    string  $UF    sigla da unidade da Federação
     * @param int|string $tpAmb tipo de ambiente 1-produção e 2-homologação
     * @param string     $modSOAP
     * @internal param $integer 1 usa o __sendSOAP e 2 usa o __sendSOAP2
     * @return    mixed False ou array ['bStat'=>boolean,'cStat'=>107,'tMed'=>1,'dhRecbto'=>'12/12/2009','xMotivo'=>'Serviço em operação','xObs'=>'']
     */
    public function statusServico($UF = '', $tpAmb = '', $modSOAP = '2', $svan = false)
    {
        //retorno da funçao
//        $aRetorno = array('bStat' => false, 'cStat' => '', 'tMed' => '', 'dhRecbto' => '', 'xMotivo' => '', 'xObs' => '');
        $aRetorno = false;

        // caso o parametro tpAmb seja vazio
        if (empty($tpAmb)) {
            $tpAmb = $this->tpAmb;
        }
        $aURL = $this->aURL;
        // caso a sigla do estado esteja vazia
        if (empty($UF)) {
            $UF = $this->UF;
        } else {
            //            if ($UF != $this->UF){
            if ($svan) {
                //recarrega as url referentes aos dados passados como parametros para a função
                $aURL = $this->loadSEFAZ($this->raizDir . 'config' . DIRECTORY_SEPARATOR . "def_ws2.xml", $tpAmb, $UF, true);
            }
        }

        //busca o cUF
        $cUF = $this->cUFlist[$UF];
        //identificação do serviço
        $servico = 'NfeStatusServico';
        //recuperação da versão
        $versao = $aURL[$servico]['version'];
        //recuperação da url do serviço
        $urlservico = $aURL[$servico]['URL'];
        //recuperação do método
        $metodo = $aURL[$servico]['method'];
        //montagem do namespace do serviço
        $namespace = $this->URLPortal . '/wsdl/' . $servico . '2';

        //montagem do cabeçalho da comunicação SOAP
        $cabec = '<nfeCabecMsg xmlns="' . $namespace . '"><cUF>' . $cUF . '</cUF><versaoDados>' . $versao . '</versaoDados></nfeCabecMsg>';
        //montagem dos dados da mensagem SOAP
        $dados = '<nfeDadosMsg xmlns="http://www.portalfiscal.inf.br/nfe/wsdl/NfeStatusServico2"><consStatServ xmlns="' . $this->URLPortal . '" versao="' . $versao . '"><tpAmb>' . $tpAmb . '</tpAmb><cUF>' . $cUF . '</cUF><xServ>STATUS</xServ></consStatServ></nfeDadosMsg>';

        if ($modSOAP == '2') {
            $retorno = $this->__sendSOAP2($urlservico, $namespace, $cabec, $dados, $metodo, $tpAmb);
        } else {
            $retorno = $this->__sendSOAP($urlservico, $namespace, $cabec, $dados, $metodo, $tpAmb, $UF);
        }

        //verifica o retorno do SOAP
        if ($retorno) {
            //tratar dados de retorno
            $doc                     = new \DOMDocument(); //cria objeto DOM
            $doc->formatOutput       = false;
            $doc->preserveWhiteSpace = false;
            $doc->loadXML($retorno, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
            $cStat = !empty($doc->getElementsByTagName('cStat')->item(0)->nodeValue) ? $doc->getElementsByTagName('cStat')->item(0)->nodeValue : '';
            if ($cStat == '') {
                //houve erro
                return false;
            } else {
                if ($cStat == '107') {
                    $aRetorno['bStat'] = true;
                }
            }
            // status do serviço
            $aRetorno['cStat'] = $doc->getElementsByTagName('cStat')->item(0)->nodeValue;
            // tempo medio de resposta
            $aRetorno['tMed'] = $doc->getElementsByTagName('tMed')->item(0)->nodeValue;
            // data e hora da mensagem (opcional)
            $aRetorno['dhRecbto'] = !empty($doc->getElementsByTagName('dhRecbto')->item(0)->nodeValue) ? date("d/m/Y H:i", $this->__convertTime($doc->getElementsByTagName('dhRecbto')->item(0)->nodeValue)) : '';
            // motivo da resposta (opcional)
            $aRetorno['xMotivo'] = !empty($doc->getElementsByTagName('xMotivo')->item(0)->nodeValue) ? $doc->getElementsByTagName('xMotivo')->item(0)->nodeValue : '';
            // obervaçoes (opcional)
            $aRetorno['xObs'] = !empty($doc->getElementsByTagName('xObs')->item(0)->nodeValue) ? $doc->getElementsByTagName('xObs')->item(0)->nodeValue : '';
        } elseif(!isset($retorno)) {
            $this->errStatus = true;
            $this->errMsg    = 'Nao houve retorno Soap verifique a mensagem de erro e o debug!!';
            $aRetorno        = false;
        }
        return $aRetorno;
    } //fim cancEvent

    public function cancelEvent($chNFe = '', $nProt = '', $xJust = '', $tpAmb = '', $metodoTransmissao = 'SEFAZ', $modSOAP = '2')
    {
        try {
            //validaÃ§Ã£o dos dados de entrada
            if ($chNFe == '' || $nProt == '' || $xJust == '') {
                $msg = "NÃ£o foi passado algum dos parÃ¢metros necessÃ¡rios ID=$chNFe ou protocolo=$nProt ou justificativa=$xJust.";
                throw new \Exception($msg);
            }
            if ($tpAmb == '') {
                $tpAmb = $this->tpAmb;
            }
            if (strlen($xJust) < 15) {
                $msg = "A justificativa deve ter pelo menos 15 digitos!!";
                throw new \Exception($msg);
            }
            if (strlen($xJust) > 255) {
                $msg = "A justificativa deve ter no mÃ¡ximo 255 digitos!!";
                throw new \Exception($msg);
            }
            if (strlen($chNFe) != 44) {
                $msg = "Uma chave de NFe vÃ¡lida nÃ£o foi passada como parÃ¢metro $chNFe.";
                throw new \Exception($msg);
            }
            //estabelece o codigo do tipo de evento CANCELAMENTO
            $tpEvento   = '110111';
            $descEvento = 'Cancelamento';
            //para cancelamento o numero sequencia do evento sempre serÃ¡ 1
            $nSeqEvento = '1';
            //remove qualquer caracter especial
            $xJust = $this->__cleanString($xJust);
            //decompor a chNFe e pegar o tipo de emissÃ£o
            $tpEmiss = substr($chNFe, 34, 1);
            /*//verifica se o SCAN esta habilitado
            if (!$this->enableSCAN){
                $aURL = $this->aURL;
            } else {
                $aURL = $this->loadSEFAZ( $this->raizDir . 'config' . DIRECTORY_SEPARATOR . $this->xmlURLfile,$tpAmb,'SCAN');
            }*/
            if ($metodoTransmissao == 'SEFAZ') {
                $aURL = $this->aURL;
            } elseif ($metodoTransmissao == 'SVAN') {
                $aURL = $this->loadSEFAZ($this->raizDir . 'config' . DIRECTORY_SEPARATOR . $this->xmlURLfile, $this->config["ambiente"], $this->config["UF"], true);
            }
            $numLote = substr(str_replace(',', '', number_format(microtime(true) * 1000000, 0)), 0, 15);
            //Data e hora do evento no formato AAAA-MM-DDTHH:MM:SSTZD (UTC)
            $dhEvento = date('Y-m-d') . 'T' . date('H:i:s') . $this->timeZone;
            //se o envio for para svan mudar o numero no orgÃ£o para 91
//            if ($this->enableSVAN) {
//                $cOrgao = '90';
//            } else {
                $cOrgao = $this->cUF;
//            }
            //montagem do namespace do serviÃ§o
            $servico = 'RecepcaoEvento';
            //recuperaÃ§Ã£o da versÃ£o
            $versao = $aURL[$servico]['version'];
            //recuperaÃ§Ã£o da url do serviÃ§o
            $urlservico = $aURL[$servico]['URL'];
            //recuperaÃ§Ã£o do mÃ©todo
            $metodo = $aURL[$servico]['method'];
            //montagem do namespace do serviÃ§o
            $namespace = $this->URLPortal . '/wsdl/' . $servico;
            //de acordo com o manual versÃ£o 5 de marÃ§o de 2012
            // 2   +    6     +    44         +   2  = 54 digitos
            //â€œIDâ€ + tpEvento + chave da NF-e + nSeqEvento
            //garantir que existam 2 digitos em nSeqEvento para montar o ID com 54 digitos
            if (strlen(trim($nSeqEvento)) == 1) {
                $zenSeqEvento = str_pad($nSeqEvento, 2, "0", STR_PAD_LEFT);
            } else {
                $zenSeqEvento = trim($nSeqEvento);
            }
            $id = "ID" . $tpEvento . $chNFe . $zenSeqEvento;
            //monta mensagem
            $Ev = '';
            $Ev .= "<evento xmlns=\"$this->URLPortal\" versao=\"$versao\">";
            $Ev .= "<infEvento Id=\"$id\">";
            $Ev .= "<cOrgao>$cOrgao</cOrgao>";
            $Ev .= "<tpAmb>$tpAmb</tpAmb>";
            $Ev .= "<CNPJ>$this->cnpj</CNPJ>";
            $Ev .= "<chNFe>$chNFe</chNFe>";
            $Ev .= "<dhEvento>$dhEvento</dhEvento>";
            $Ev .= "<tpEvento>$tpEvento</tpEvento>";
            $Ev .= "<nSeqEvento>$nSeqEvento</nSeqEvento>";
            $Ev .= "<verEvento>$versao</verEvento>";
            $Ev .= "<detEvento versao=\"$versao\">";
            $Ev .= "<descEvento>$descEvento</descEvento>";
            $Ev .= "<nProt>$nProt</nProt>";
            $Ev .= "<xJust>$xJust</xJust>";
            $Ev .= "</detEvento></infEvento></evento>";
            //assinatura dos dados
            $tagid = 'infEvento';
            $Ev    = $this->signXML($Ev, $tagid);
            $Ev    = str_replace('<?xml version="1.0"?>', '', $Ev);
            $Ev    = str_replace('<?xml version="1.0" encoding="utf-8"?>', '', $Ev);
            $Ev    = str_replace('<?xml version="1.0" encoding="UTF-8"?>', '', $Ev);
            $Ev    = str_replace(array("\r", "\n", "\s"), "", $Ev);
            //carrega uma matriz temporÃ¡ria com os eventos assinados
            //montagem dos dados
            $dados = '';
            $dados .= "<envEvento xmlns=\"$this->URLPortal\" versao=\"$versao\">";
            $dados .= "<idLote>$numLote</idLote>";
            $dados .= $Ev;
            $dados .= "</envEvento>";
            //montagem da mensagem
            $cabec = "<nfeCabecMsg xmlns=\"$namespace\"><cUF>$this->cUF</cUF><versaoDados>$versao</versaoDados></nfeCabecMsg>";
            $dados = "<nfeDadosMsg xmlns=\"$namespace\">$dados</nfeDadosMsg>";
            //grava solicitaÃ§Ã£o em temp
            $arqName = $this->temDir . "$chNFe-$nSeqEvento-eventCanc.xml";
            if (!file_put_contents($arqName, $Ev)) {
                $msg = "Falha na gravaÃ§Ã£o do arquivo $arqName";
                $this->__setError($msg);
            }
            //envia dados via SOAP
            if ($modSOAP == '2') {
                $retorno = $this->__sendSOAP2($urlservico, $namespace, $cabec, $dados, $metodo, $tpAmb);
            } else {
                $retorno = $this->__sendSOAP($urlservico, $namespace, $cabec, $dados, $metodo, $tpAmb, $this->UF);
            }
            //verifica o retorno
            if (!$retorno) {
                //nÃ£o houve retorno
                $msg = "Nao houve retorno Soap verifique a mensagem de erro e o debug!!";
                throw new \Exception($msg);
            }
            //tratar dados de retorno
            $xmlretEvent                     = new \DOMDocument('1.0', 'utf-8'); //cria objeto DOM
            $xmlretEvent->formatOutput       = false;
            $xmlretEvent->preserveWhiteSpace = false;
            $xmlretEvent->loadXML($retorno, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
            $retEnvEvento = $xmlretEvent->getElementsByTagName("retEnvEvento")->item(0);
            $cStat        = !empty($retEnvEvento->getElementsByTagName('cStat')->item(0)->nodeValue) ? $retEnvEvento->getElementsByTagName('cStat')->item(0)->nodeValue : '';
            $xMotivo      = !empty($retEnvEvento->getElementsByTagName('xMotivo')->item(0)->nodeValue) ? $retEnvEvento->getElementsByTagName('xMotivo')->item(0)->nodeValue : '';
            if ($cStat == '') {
                //houve erro
                $msg = "cStat estÃ¡ em branco, houve erro na comunicaÃ§Ã£o Soap verifique a mensagem de erro e o debug!!";
                throw new \Exception($msg);
            }
            //tratar erro de versÃ£o do XML
            if ($cStat == '238' || $cStat == '239') {
                $this->__trata239($retorno, $this->UF, $tpAmb, $servico, $versao);
                $msg = "VersÃ£o do arquivo XML nÃ£o suportada no webservice!!";
                throw new \Exception($msg);
            }
            //erro no processamento cStat <> 128
            if ($cStat != 128) {
                //se cStat <> 135 houve erro e o lote foi rejeitado
                $msg = "Retorno de ERRO: $cStat - $xMotivo";
                throw new \Exception($msg);
            }
            //o lote foi processado cStat == 128
            $retEvento = $xmlretEvent->getElementsByTagName("retEvento")->item(0);
            $cStat     = !empty($retEvento->getElementsByTagName('cStat')->item(0)->nodeValue) ? $retEvento->getElementsByTagName('cStat')->item(0)->nodeValue : '';
            $xMotivo   = !empty($retEvento->getElementsByTagName('xMotivo')->item(0)->nodeValue) ? $retEvento->getElementsByTagName('xMotivo')->item(0)->nodeValue : '';


            if ($cStat != self::CANC_NFE_HOMOLOGADO && $cStat != self::EVENTO_REG_VINC_NFE &&
                $cStat != self::CANC_NFE_HOM_FORA_DO_PRAZO && $cStat != self::EVENTO_REG_N_VINC_NFE &&
                $cStat != self::CANC_REC_EVENTO
            ) {
                $msg = "Retorno de ERRO: $cStat - $xMotivo";
                throw new \Exception($msg);
            }
            //o evento foi aceito
            //carregar o evento
            $xmlenvEvento                     = new \DOMDocument('1.0', 'utf-8'); //cria objeto DOM
            $xmlenvEvento->formatOutput       = false;
            $xmlenvEvento->preserveWhiteSpace = false;
            $xmlenvEvento->loadXML($Ev, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG);
            $evento = $xmlenvEvento->getElementsByTagName("evento")->item(0);
            //Processo completo solicitaÃ§Ã£o + protocolo
            $xmlprocEvento = new \DOMDocument('1.0', 'utf-8');; //cria objeto DOM
            $xmlprocEvento->formatOutput       = false;
            $xmlprocEvento->preserveWhiteSpace = false;
            //cria a tag procEventoNFe
            $procEventoNFe = $xmlprocEvento->createElement('procEventoNFe');
            $xmlprocEvento->appendChild($procEventoNFe);
            //estabele o atributo de versÃ£o
            $eventProc_att1 = $procEventoNFe->appendChild($xmlprocEvento->createAttribute('versao'));
            $eventProc_att1->appendChild($xmlprocEvento->createTextNode($versao));
            //estabelece o atributo xmlns
            $eventProc_att2 = $procEventoNFe->appendChild($xmlprocEvento->createAttribute('xmlns'));
            $eventProc_att2->appendChild($xmlprocEvento->createTextNode($this->URLPortal));
            //carrega o node evento
            $node1 = $xmlprocEvento->importNode($evento, true);
            $procEventoNFe->appendChild($node1);
            //carrega o node retEvento
            $node2 = $xmlprocEvento->importNode($retEvento, true);
            $procEventoNFe->appendChild($node2);
            //salva o xml como string em uma variÃ¡vel
            $procXML = $xmlprocEvento->saveXML();
            //remove as informaÃ§Ãµes indesejadas
            $procXML = str_replace("xmlns:default=\"http://www.w3.org/2000/09/xmldsig#\"", '', $procXML);
            $procXML = str_replace('default:', '', $procXML);
            $procXML = str_replace(':default', '', $procXML);
            $procXML = str_replace("\n", '', $procXML);
            $procXML = str_replace("\r", '', $procXML);
            $procXML = str_replace("\s", '', $procXML);
            //salva o arquivo xml
            $arqName = $this->canDir . "$chNFe-$nSeqEvento-procCanc.xml";
            if (!file_put_contents($arqName, $procXML)) {
                $msg = "Falha na gravaÃ§Ã£o do arquivo $arqName";
                throw new \Exception($msg);
            }
        } catch (\Exception $e) {
            throw $e;
            return false;
        }
        return $procXML;
    } //fim getProtocol
}