<?php

/* Registro40GRRF.
 * @package Logic
 * @author  francismararaujo
 */

class Registro40GRRF extends RegistroGRRF{

    const REGISTRO_40 = '40';
    const SIM = 'S';
    const NAO = 'N';
    const TRABALHADO = '1';
    const INDENIZADO = '2';
    const DISPENSA = '3';
    const PENSAOJUDICIAL = '3';

    private $_pessoa;
    private $_vinculo;
    private $_escalaSemanal;
    private $_tpInscricao;
    private $_nrInscricao;
    private $_tpInscricaoTomador;
    private $_nrInscricaoTomador;
    private $_nrRecolhimento;
    private $_nrCategoria;
    private $_competencia;
    private $_competencia13;
    private $_codMovimentacao;
    private $_codSaque;
    private $_banco;
    private $_agencia;
    private $_contacorrente;
    private $_tpDependente;
    private $_dtHomologacao;
    private $_fgtsmesanterior;

    /* Preparacao para a criacao do Registro 40.
     * @param GpepessoaEntity   $pessoa          Instancia de GpepessoaEntity.
     * @param GpeVinculomEntity $vinculo         Instancia de GpeVinculomEntity.
     * @param string            $remuneracao     Valor da remuneracao do funcionario.
     * @return void
     * @author francismararaujo
     */

    public function geraRegistro40(GpepessoaEntity $pessoa, GpeVinculomEntity $vinculo, $avisoprevioindenizado,
                                   $mesanteriorrescisao, $mesrescisao, $pensaojudicial, $saldofinsrescisorios){

        $this->_pessoa = $pessoa;
        $this->_vinculo = $vinculo;
        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_competencia = substr($this->getCompetencia(), 4, 2);
        $this->_competencia13 =  $this->getCompetencia13();
        $this->_geraRegistro($avisoprevioindenizado, $mesanteriorrescisao, $mesrescisao, $pensaojudicial,
            $saldofinsrescisorios);

    }

    /* Cria o Registro 40.
     * @param string $remuneracao Valor da remuneracao do funcionario.
     * @return void
     * @author francismararaujo
     */

    private function _geraRegistro($avisoprevioindenizado, $mesanteriorrescisao, $mesrescisao, $pensaojudicial, $saldofinsrescisorios){

        //bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_40, $this->_tpInscricao, $this->_nrInscricao);

        //bloco 4 | 018 | tipo de inscricao do tomador.
        $this->atribuiDado($this->_tpInscricaoTomador, 1, parent::CARACTER_ZERO, parent::PREENCHE_DIREITA);

        //bloco 5 | 019-032 | inscricao do tomador.
        $this->atribuiDado($this->_nrInscricaoTomador, 14, parent::CARACTER_ZERO, parent::PREENCHE_DIREITA);

        //bloco 6 | 033-043 | pis/pasep/ci.
        $identificador = $this->_pessoa->historico->getNrpispaseppes();
        $this->atribuiDado($identificador, 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 7 | 044-051 | data de admissao.
        $this->_atribuiDataAdmissao($this->_vinculo->getDtadmissaovinc());

        //bloco 8 | 052-053 | categoria do trabalahador.
        //@todo rever campos de categoria na situacao funcional e no vinculo
        $this->atribuiDado($this->_nrCategoria, 2);

        //bloco 9 | 054-123 | nome do trabalhador.
        $nome = $this->_pessoa->historico->getNmpessoa();
        $this->atribuiDado($nome, 70, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 10 | 124-130 | numero ctps.
        $this->_atribuiCTPS($this->_pessoa->historico->getNrctpspessoa());

        //bloco 11 | 131-135 | Serie ctps.
        $this->_atribuiSerieCTPS($this->_pessoa->historico->getNrseriectpspes());

        //bloco 12 | 136 | sexo (1 - masculino / 2 - feminino).
        $this->_idSexoPessoa($this->_pessoa->historico->getIdSexoPessoa());

        //bloco 13 | 137-138 | grau de instrucao.
        $nrgrauinstr = $this->_pessoa->historico->getNrGrauInstr();
        $this->atribuiDado($nrgrauinstr, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 14 | 139-146 | data de nascimento.
        $this->_atribuiDataNascimento($this->_pessoa->historico->getDtnascpessoa());

        //bloco 15 | 147-148 | qt horas trabalhadas por semana.
        $this->atribuiDado($this->_escalaSemanal, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 16 | 149-154 | Codigo Brasileiro de Ocupacao.
        $this->_atribuiCbo($this->_vinculo->cbo);

        //bloco 17 | 155-162 | data de opcao.
        $this->_atribuiDataAdmissao($this->_vinculo->getDtadmissaovinc());

        //bloco 18 | 163-164 | codigo de movimentacao.
        $this->_atribuiCodigoMovimentacao();

        //bloco 19 | 165-172 | data de movimentacao.
        $this->_atribuiDataRescisao($this->_vinculo->getDtrescisaovinc());

        //bloco 20 | 173-175 | codigo saque.
        $this->_atribuiCodigoSaque($this->_codSaque);

        //bloco 21 | 176 | aviso previo.
        $avisoprevio = $this->_retornaAvisoPrevio();
        $this->atribuiDado($avisoprevio, 1, parent::CARACTER_ZERO, parent::PREENCHE_DIREITA);

        //bloco 22 | 177-184 | data inicio do aviso previo.
        $this->_atribuiDataAvisoPrevio();

        //bloco 23 | 185 | reposicao de vaga.
        //@todo default 'S' conferir de onde vem esta informacao
        $this->atribuiDado('S', 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 24 | 186-193 | data da homologacao.
//        $this->atribuiDado(parent::CARACTER_BRANCO, 8, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        $this->_atribuiDataHomologacao();

        //bloco 25 | 194-208 | valor dissidio.
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//        $this->_atribuiValorDissidio($remuneracao[RegistroGRRF::GRRF_13SAL_PAGO_MES]);

        //bloco 26 | 209-223 | remuneracao - mes anterior a rescisao.
//        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $this->_atribuiRemuneracaoMesAnteriorRescisao($mesanteriorrescisao[RegistroGRRF::GRRF_MES_ANTERIOR_RESCISAO]);

        //bloco 27 | 224-238 | remuneracao - mes da rescisao
        $this->_atribuiRemuneracao13($mesrescisao[RegistroGRRF::GRRF_MES_RESCISAO]);

        //bloco 28 | 239-253 | aviso previo indenizado.

        $this->_atribuiAvisoPrevioIndenizado($avisoprevioindenizado[RegistroGRRF::GRRF_AVISO_PREVIO_INDENIZADO]);

        //bloco 29 | 254 | indicativo pensao alimenticia.
        $this->atribuiDado($this->_atribuiPensaoAlimenticia(), 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 30 | 255-259 | percentual de pensao alimenticia.
        $this->atribuiDado(parent::CARACTER_ZERO, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 31 | 260-274 | valor de pensao alimenticia.
        $this->_atribuiPensaoJuridica($pensaojudicial[RegistroGRRF::GRRF_PENSAO_JURIDICA]);

        //bloco 32 | 275-285 | cpf.
        $cpf = $this->_pessoa->historico->getNrcpfpessoa();
        $this->atribuiDado($cpf, 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 33 | 286-288 | banco da conta do trabalhador.
        //$this->atribuiDado($this->_banco, 3, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $this->atribuiDado(parent::CARACTER_ZERO, 3, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 34 | 289-292 | agencia da conta do trabalhador.
        //$this->atribuiDado($this->_agencia, 4, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $this->atribuiDado(parent::CARACTER_ZERO, 4, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 35 | 293-305 | numero da conta bancaria do trabalhador.
        //$this->atribuiDado($this->_contacorrente, 13, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $this->atribuiDado(parent::CARACTER_ZERO, 13, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 36 | 306-320 | saldo para fins rescisorios.
        $this->_atribuiDadoSaldoRescisorio($saldofinsrescisorios[RegistroGRRF::GRRF_SALDO_FINS_RESCISORIOS]);

        //bloco 37 | 321-359 | preencher com brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 39, parent::CARACTER_BRANCO);

        //bloco 38 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);

    }

    public function setFGTSMesAnterior($fgtsmesanterior)
    {
        $this->_fgtsmesanterior = $fgtsmesanterior;
    }

    public function setDtHomologacao($dtHomologacao)
    {
        $this->_dtHomologacao = $dtHomologacao;
    }

    public function setCodMovimentacao($codMovimentacao)
    {
        $this->_codMovimentacao = $codMovimentacao;
    }

    public function setCodSaque($codsaque)
    {
        $this->_codSaque = $codsaque;
    }

    public function setNrCategoria($nrCategoria)
    {
        $this->_nrCategoria = $nrCategoria;
    }

    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }

    public function setNrInscricaoTomador($nrInscricaoTomador)
    {
        $this->_nrInscricaoTomador = $nrInscricaoTomador;
    }

    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }

    public function setTpInscricaoTomador($tpInscricaoTomador)
    {
        $this->_tpInscricaoTomador = $tpInscricaoTomador;
    }

    public function setEscalaSemanal($escalaSemanal)
    {
        $this->_escalaSemanal = $escalaSemanal;
    }

    public function setBanco($banco)
    {
        $this->_banco = $banco;
    }

    public function setAgencia($agencia)
    {
        $this->_agencia = $agencia;
    }

    public function setContaCorrente($contacorrente)
    {
        $this->_contacorrente = $contacorrente;
    }

    public function setTipoDependente($tpdependente)
    {
        $this->_tpDependente = $tpdependente;
    }

    private function _idSexoPessoa($idSexoPessoa){

        if($idSexoPessoa == 'M'){
            $this->atribuiDado('1', 1, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado('2', 1, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        }

    }

//    /* Atribui Valor Dissídio.
//    * @return void
//    * @author francismararaujo - 02/05/2013
//    */
//
//    private function _atribuiValorDissidio($valorDissidio){
//
////        Campo obrigatório se o campo Data da Homologação do Dissídio estiver preenchido.
////        Informar o valor do FGTS referente ao dissídio.
////        Se informado deve ter 2 casas decimais válidas.
////        Sempre que não informado preencher com zeros.
//
//        if($this->_dtHomologacao){
//            $this->atribuiDado($valorDissidio, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//            unset($valorDissidio);
//        } else {
//            $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//            unset($valorDissidio);
//        }
//
//    }

    /* Atribui Valor Dissídio.
    * @return void
    * @author francismararaujo - 02/05/2013
    */

    private function _atribuiDadoSaldoRescisorio($saldofinsrescisorios){

//    Campo Opcional.
//    Valor do saldo para fins rescisórios informado pela empresa que será base de cálculo da multa rescisória.
//    Esse campo deverá ser preenchido com zeros quando informado valor no campo “Valor Dissídio”.

        if($this->_dtHomologacao){
            $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado($saldofinsrescisorios, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

    }

    /* Atribui Aviso Prévio Indenizado.
     * @return void
     * @author francismararaujo - 05/04/2013
     */

    private function _atribuiAvisoPrevioIndenizado($avisoprevioindenizado){

        $avisoprevio = array('02');

        if (array_search($this->_retornaAvisoPrevio(), $avisoprevio) !== false) {

            $this->atribuiDado($avisoprevioindenizado, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        }else{

            $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        }

    }

    /* Atribui Pensao Judicial.
     * @return void
     * @author francismararaujo - 06/04/2013
     */

    private function _atribuiPensaoJuridica($pensaojuridica){

        $pensaoalimenticia = array('V');
        if (array_search($this->_atribuiPensaoAlimenticia(), $pensaoalimenticia) !== false) {
            $this->atribuiDado($pensaojuridica, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

    }

    /* Atribui Pensao Alimenticia.
     * @return void
     * @author francismararaujo - 01/03/2013
     */

    private function _atribuiPensaoAlimenticia(){

        //campo obrigatório, exceto para tipo de remessa igual a 5.
        //deve ser igual a N (Não existe Pensão Alimentícia) ou V (Valor da Pensão Alimentíci

        if($this->_tpDependente){

            foreach($this->_tpDependente as $dependente){

                if($dependente->getNrtipodepende() == self::PENSAOJUDICIAL){
                    $tpdependente= 'V';
                }else{
                    $tpdependente= 'N';
                }

            }

        }else{
            $tpdependente= 'N';
        }

//        @todo - NUTRI/ALIBRAS usa padrao 'N' verificar se é necessário informar pensão alimentícia para GRRF.
        //return $tpdependente;
        return 'N';

    }

    /* Atribui Aviso Previo.
     * @param string $dtAvisoPrevio Data Aviso Previo.
     * @return void
     * @author francismararaujo - 28/02/2013
     */

    private function _retornaAvisoPrevio(){

        /* Campo obrigatório, exceto para tipo de remessa igual a 5.
         * 1 – Trabalhado;
         * 2 – Indenizado;
         * 3 – Ausência/dispensa;
         * Deverá ser preenchido com 1 ou 3 para Código de Movimentação I3.
         */

        if($this->_vinculo->getDtavisoprevio() !== false){

            $dtavisoprevio = new TkDateTime($this->_vinculo->getDtavisoprevio());
            $dtrescisao = new TkDateTime($this->_vinculo->getDtrescisaovinc());

            if($dtavisoprevio->isEarlier($dtrescisao)){
                $avisoprevio = self::TRABALHADO;
            }

            if($dtavisoprevio->equals($dtrescisao)){
                $avisoprevio = self::INDENIZADO;
            }

            return $avisoprevio;

        }else{

            return self::DISPENSA;

        }


    }

    /* Atribui Codigo Saque.
     * @param string $nrtpdemissao Numero tipo de demissao.
     * @return void
     * @author francismararaujo - 28/02/2013
     */

    private function _atribuiCodigoSaque($codsaque){

        /* Campo obrigatório, exceto para tipo de remessa igual a 5.
         * Só pode ser 01, 02, 03 ou 04 (06, 05 e 05A – implementação futura).
         * Para tipo de remessa igual a 2, aceitar 01,02, 03, 04.
         * Se código de movimentação = H,J ou M, este campo deve ficar em branco.
         * Campo deve ser alinhado à esquerda.
         *
         */

        $codmovimentacao = array('H','J','M');
        if (array_search($this->_codMovimentacao, $codmovimentacao) !== false) {

            $this->atribuiDado(parent::CARACTER_BRANCO, 3, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
            unset($nrTpDemissao);

        }else{

            //$this->atribuiDado(str_repeat('0',2 - strlen($codsaque)) . $codsaque, 3, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            $this->atribuiDado($codsaque, 3, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            unset($nrTpDemissao);

        }

    }

    /* Atribui Data de Homologacao.
    * @return void
    * @author francismararaujo - 02/05/2013
    */

    private function _atribuiDataHomologacao(){

//        Campo Opcional.
//        Deve ser uma data válida .
//        Formato DDMMAAAA

        if($this->_dtHomologacao){
            $dataHomologacao = new TkDateTime($this->_dtHomologacao);
            $dataHomologacao = $dataHomologacao->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
        } else {
            $dataHomologacao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataHomologacao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataHomologacao);
    }

    /* Atribui Data de Admissao.
     * @param string $dtAdmissao Data de Admissao.
     * @return void
     * @author francismararaujo - 26/02/2013
     */

    private function _atribuiDataAdmissao($dtAdmissao){

        //obrigatorio para as categorias de trabalhadores 01, 03, 04, 05, 06, 07, 11, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '03', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
            $dataAdmissao = new TkDateTime($dtAdmissao);
            $dataAdmissao = $dataAdmissao->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
        } else {
            $dataAdmissao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataAdmissao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataAdmissao);

    }

    /* Atribui Data de Nascimento.
     * @param string $dtNascimento Data de Nascimento do Funcionario.
     * @return void
     * @author francismararaujo - 27/02/2013
     */

    private function _atribuiDataNascimento($dtNascimento){

        // Campo obrigatorio para as categorias de trabalhadores 01, 02, 03, 04, 05, 06, 07, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '02', '03', '04', '05', '06', '07', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $dataNascimento = parent::CARACTER_BRANCO;
        } else {
            $dataNascimento = new TkDateTime($dtNascimento);
            $dataNascimento = $dataNascimento->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
        }

        $this->atribuiDado($dataNascimento, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataNascimento);

    }

    /* Atribui CTPS.
     * @param string $nrCTPS Numero da Carteira Profissional.
     * @return void
     * @author francismararaujo - 26/02/2013
     */

    private function _atribuiCTPS($nrCTPS){

        /* Obrigatorio para as categorias de trabalhadores 01, 03 , 04, 06, 07 e 26.
         * Opcional para a categoria de trabalhador 02.
         * Não pode ser informado para as demais categorias.
         * Sempre que não informado o campo deve ficar em branco.
         */

        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $nrCTPS = parent::CARACTER_BRANCO;
        }

        if ((empty($nrCTPS) === true) || ($nrCTPS === parent::CARACTER_BRANCO)) {
            $this->atribuiDado($nrCTPS, 7, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        } else {
            $this->atribuiDado($nrCTPS, 7, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

    }

    /* Atribui Serie CTPS.
     * @param string $serieCTPS Serie da Carteira Profissional.
     * @return void
     * @author francismararaujo - 26/02/2013
     */

    private function _atribuiSerieCTPS($serieCTPS){
        /* Obrigatorio para as categorias de trabalhadores 01, 03 , 04, 06, 07 e 26.
         * Opcional para a categoria de trabalhador 02.
         */
        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $serieCTPS = parent::CARACTER_BRANCO;
        }

        if ((empty($serieCTPS) === true) || ($serieCTPS === parent::CARACTER_BRANCO)) {
            $this->atribuiDado($serieCTPS, 5, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        } else {
            $this->atribuiDado($serieCTPS, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

    }

    /* Atribui codigo CBO.
     * @param string $cbo Codigo CBO.
     * @return void
     * @author francismararaujo - 27/02/2013
     */

    private function _atribuiCbo($cbo){

        /* Utilizar os quatro primeiros dígitos do grupo “Família” do novo CBO, acrescentando zero a
         * esquerda.(0 + XXXX onde XXXX é o código da família do novo CBO a qual pertence o
         * trabalhador).
         */

        if (strlen($cbo) > 4){
            $cbo = substr($cbo, 0, 4);
        }

        $this->atribuiDado($cbo, '6', parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

    }

    /* atribui codigo movimentação.
    * @return void
    * @author francismararaujo - 03/04/2013
    */

    private function _atribuiCodigoMovimentacao(){

        /* Campo obrigatório, exceto para tipo de remessa igual a 5.
         * Acatarsomente I1,I2, I3, I4 ou L.
         * Não pode acatar código de movimentação I4 para categoria diferente de 06.
         * Não pode acatar código de movimentação I1 para categoria 06.
         * Acatar H,J ou M se tipo de remessa igual a 4.
         * S1, S2, U1, U2 e U3, H,J ou M para implementação futura.
         */

        $codigomovimentacao = array('I1','I2','I3','I4','L');
        if(array_search($this->_codMovimentacao,$codigomovimentacao) !== false){
            $this->atribuiDado($this->_codMovimentacao, '2', parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado(parent::CARACTER_ZERO, '2', parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }


    }

    /* atribui data de recisao.
     * @param string $dtRescisaoVinc data de recisao do vinculo.
     * @return void
     * @author francismararaujo - 27/02/2013
     */

    private function _atribuiDataRescisao($dtRescisao){

        //obrigatorio para as categorias de trabalhadores 01, 03, 04, 05, 06, 07, 11, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '03', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
            $dataRecisao = new TkDateTime($dtRescisao);
            $dataRecisao = $dataRecisao->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
        } else {
            $dataRecisao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataRecisao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataRecisao);

    }

    /* atribui data Aviso Previo.
    * @param string $data.
    * @return void
    * @author francismararaujo - 27/02/2013
    */

    private function _atribuiDataAvisoPrevio(){

        /* Campo obrigatório se campo “Aviso Prévio” igual a “1” ou “3”
         * Formato DDMMAAAA.
         * Deve ser maior ou igual a data de Admissão e menor ou igual a data de Movimentação.
         * Não deve ser preenchido se Aviso Prévio for igual a 2 – Indenizado.
         */

        $avisoprevio = array('01', '03');

        if ((array_search($this->_retornaAvisoPrevio(), $avisoprevio)
            && ($this->_vinculo->getDtavisoprevio() !== false))) {

            $dtavisoprevio = new TkDateTime($this->_vinculo->getDtavisoprevio());
            $dtrescisao = new TkDateTime($this->_vinculo->getDtrescisaovinc());
            $dtadmissao = new TkDateTime($this->_vinculo->getDtadmissaovinc());

            if(($dtavisoprevio->isLater($dtadmissao) || $dtavisoprevio->equals($dtadmissao)) && ($dtavisoprevio->isEarlier($dtrescisao) || $dtavisoprevio->equals($dtrescisao))){

                $dataFormatada = new TkDateTime($dtavisoprevio);
                $dataFormatada = $dataFormatada->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
                $this->atribuiDado($dataFormatada, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
                unset($dataFormatada);

            }

        }else{

            $this->atribuiDado(parent::CARACTER_BRANCO, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            unset($dataFormatada);

        }

    }


//    /**
//     * Atribui Matricula.
//     *
//     * @param string $matricula Matricula.
//     *
//     * @return void
//     *
//     * @author alessandrobartels - 01/02/2012
//     */
//    private function _atribuiMatricula($matricula){
//
//        // Nao pode ser informado para as categorias 06, 13, 14, 15, 16 , 17, 18, 22, 23, 24 ou 25.
//        $codCategorias = array(
//            '06', '13', '14', '15', '16', '17', '18', '22', '23', '24', '25',
//        );
//
//        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
//            $matricula = parent::CARACTER_BRANCO;
//        }
//
//        if ((empty($matricula) === true) || ($matricula === parent::CARACTER_BRANCO)) {
//            $this->atribuiDado($matricula, 11, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
//        } else {
//            $this->atribuiDado($matricula, 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//        }
//    }
//
//
//
//
//    /**
//     * Atribui Data de opcao do FGTS.
//     *
//     * @param string $dtOpcao Data de opcao do FGTS.
//     *
//     * @return void
//     *
//     * @author alessandrobartels - 01/02/2012
//     */
//    private function _atribuiDataOpcaoFGTS($dtOpcao){
//        /* Obrigat�rio para as categorias de trabalhadores 01, 03, 04 , 05, 06 e 07.
//         * N�o pode ser informado para o c�digo de recolhimento 640.
//         */
//
//        $codCategorias = array(
//            '01', '03', '04', '05', '06', '07',
//        );
//
//        if ((array_search($this->_nrCategoria, $codCategorias) === false)
//            ||($this->_nrRecolhimento === '640')
//        ) {
//            $dataOpcao = parent::CARACTER_BRANCO;
//        } else {
//            $dataOpcao = new TkDateTime($dtOpcao);
//            $dataOpcao = $dataOpcao->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
//        }
//
//        $this->atribuiDado($dataOpcao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
//        unset($dataOpcao);
//    }
//
//
//

//    /**
//     * Atribui Remuneracao sem 13.
//     *
//     * @param string $remuneracao Valor da remuneracao sem 13 salario.
//     *
//     * @return void
//     *
//     * @author alessandrobartels - 01/02/2012
//     */
//    private function _atribuiRemuneracao($remuneracao) {
//        /* Campo obrigatorio para as categorias 05, 11, 13, 14, 15, 16, 17, 18, 22, 23, 24 e 25.
//         * Opcional para as categorias 01, 02, 03, 04, 06, 07, 12, 19, 20, 21 e 26.
//         * Nao pode ser informado para a competencia 13.
//         * Obs.: Abrange todas as categorias portanto nao precisa de validacao.
//         */
//        if ($this->_competencia13 === true) {
//            $remuneracao = parent::CARACTER_ZERO;
//        } /*else {
//            $remuneracao = number_format($remuneracao, 2, '.', '');
////            $remuneracao = str_replace(self::CARACTER_PONTO, self::CARACTER_VAZIO, $remuneracao);
//        }*/
//
//        $this->atribuiDado($remuneracao, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//    }

    /* Atribui Remuneracao 13 - Parcela 13 pago no mes.
     * @param string $remuneracao13 Valor da remuneracao 13 pago no mes.
     * @return void
     * @author alessandrobartels - 01/02/2012
     */

    private function _atribuiRemuneracaoMesAnteriorRescisao($mesanteriorrescisao){

//        Campo opcional.
//        Se informado deve ter 2 casas decimais válidas.
//        Sempre que não informado preencher com zeros.
//        Não deve ser preenchido se campo Data da Homologação do Dissídio estiver preenchido.
//        Não deve ser preenchido se a Data de Admissão e a Data de Movimentação estiverem no mesmo mês.

        if(!$this->_dtHomologacao && $this->_fgtsmesanterior === 'S'){

            $dtadmissao = new TkDateTime($this->_vinculo->getDtadmissaovinc());
            $dtadmissao->toString(TkDateTime::MONTH);
            $dtmovimentacao = new TkDateTime($this->_vinculo->getDtrescisaovinc());
            $dtmovimentacao->toString(TkDateTime::MONTH);

            if($dtadmissao != $dtmovimentacao ){
                $resultado = $mesanteriorrescisao;
            }else{
                $resultado = parent::CARACTER_ZERO;
            }

        }else {
            $resultado = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($resultado, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

    }



    /* Atribui Remuneracao 13 - Parcela 13 pago no mes.
     * @param string $remuneracao13 Valor da remuneracao 13 pago no mes.
     * @return void
     * @author alessandrobartels - 01/02/2012
     */

    private function _atribuiRemuneracao13($remuneracao13){
        /* Nao pode ser informado para a competencia 13.
         * Campo obrigatorio para categoria 02.
         * Campo opcional para as categorias de trabalhadores 01, 03, 04, 06, 07, 12, 19, 20, 21 e 26.
         */
        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '12', '19', '20', '21', '26',
        );

        if ((array_search($this->_nrCategoria, $codCategorias) === false)
            ||($this->_competencia13 === true)
        ) {
            $remuneracao13 = parent::CARACTER_ZERO;
        }/* else {
            $remuneracao = number_format($remuneracao, 2, '.', '');
//            $remuneracao13 = str_replace(self::CARACTER_PONTO, self::CARACTER_VAZIO, $remuneracao13);
        }*/

        $this->atribuiDado($remuneracao13, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


//    /**
//     * Atribui Ocorrencias a agentes nocivos ou multiplos vinculos.
//     *
//     * @param string $ocorrencia Ocorrencia a agente nocivos ou multiplos vinculos.
//     *
//     * @return void
//     *
//     * @author alessandrobartels - 01/02/2012
//     */
//    private function _atribuiOcorrencia($ocorrencia){
//        /* @todo - Falta implementacao do campo para a descricao do agente nocivo.
//         * Campo opcional para as categorias 01, 02, 03, 04, 05, 06, 07, 11, 12, 13, 15, 17, 18, 19, 20, 21, 22, 23, 24 e 25.
//         * Deve ficar em branco se trabalhador não esteve exposto a agente nocivo e não possui mais de um vínculo empregatício.
//         * Para empregado doméstico (Cat 06) e diretor não empregado (Cat 05) permitido apenas branco ou 05.
//         * Para as categorias 02, 22 e 23 permitido apenas branco, 01, 02, 03 e 04.
//         * Obrigatório para categoria 26, devendo ser informado 05, 06, 07 ou 08.
//         * Não pode ser informado para as demais categorias.
//         * Deve ser uma ocorrência válida (ver tabela).
//         * Sempre que não informado o campo deve ficar em branco.
//         */
//
//        $codCategorias = array(
//            '01', '02', '03', '04', '06', '07', '11', '12', '13', '15', '17', '18', '19', '20', '21', '22', '23', '24', '25',
//        );
//
//        if (array_search($this->_nrCategoria, $codCategorias) === false) {
//            $ocorrencia = parent::CARACTER_ZERO;
//        }
//
//        $this->atribuiDado($ocorrencia, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
//
//    }
//
//
//    /**
//     * Atribui Contribuicao descontada do segurado para o INSS.
//     *
//     * @param string $contribuicao Valor da Contribuicao do segurado.
//     *
//     * @return void
//     *
//     * @author alessandrobartels - 01/02/2012
//     */
//    private function _atribuiContribuicaoINSSInformada($contribuicao){
//        /*
//         * @todo Fazer validacao da contribuicao.
//         *  Campo opcional para as categorias de trabalhadores igual a 01, 02, 04, 06, 07, 12, 19, 20, 21 e 26.
//         *  Campo opcional para as categorias de trabalhadores igual a 05, 11, 13, 15, 17, 18, 24 e 25 a partir da competência 04/2003
//         *  Campo opcional para os codigos de recolhimento 130, 135 e 650.
//         *  Campo opcional para competencia maior ou igual a 12/1999 para afastamentos por motivo de licenca-maternidade
//         *  Iniciada a partir de 01/12/1999 e com beneficios requeridos ate 31/08/2003.
//         */
//        $codCategorias = array(
//            '01', '02', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
//        );
//
//        $codMovimentacoes = array(
//            'O1', 'O2', 'R', 'Z2', 'Z3', 'Z4',
//        );
//
//        $this->atribuiDado($contribuicao, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//    }
//
//
//    /**
//     * Atribui Base GPS.
//     *
//     * @param string $baseGPS Valor da Base GPS.
//     *
//     * @return void
//     *
//     * @author alessandrobartels - 01/02/2012
//     */
//    private function _atribuiBaseGPS($baseGPS){
//        /* @todo Campo obrigatorio para as movimentacoes (registro 32) por O1, O2, R, Z2, Z3 e Z4.
//         * @todo Campo obrigatorio quando o Indicativo 'C' de Recolhimento Complementar de FGTS for informado (registro
//         *       Tipo 32 - campo 12).
//         *
//         * - Campo opcional para as categorias 01, 02, 04, 05, 06, 07,11, 12, 19, 20, 21 e 26.
//         * - Nao pode ser informado na competencia 13.
//         * - Nao pode ser informado para os codigos de recolhimento 145, 307, 317, 327, 337, 345, 640 e 660.
//         * - Sempre que nao informado preencher com zeros.
//         */
//
//        $codRecol = array(
//            '145', '307', '317', '327', '337', '345', '640', '660',
//        );
//
//        $codCategorias = array(
//            '01', '02', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
//        );
//
//        $codMovimentacoes = array(
//            'O1', 'O2', 'R', 'Z2', 'Z3', 'Z4',
//        );
//
//        if ((array_search($this->_nrRecolhimento, $codRecol) !== false)
//            ||(array_search($this->_nrCategoria, $codCategorias) === false)
//            ||(array_search($this->_codMovimentacao, $codMovimentacoes) === false)
//            ||($this->_competencia13 === true)
//        ) {
//            $baseGPS = parent::CARACTER_ZERO;
//        }
//
//        $this->atribuiDado($baseGPS, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//
//    }
//
//
//    /**
//     * Atribui Base de 13 salario.
//     *
//     * @param string $base13 Valor da Base para calculo de 13 salario.
//     *
//     * @return void
//     *
//     * @author alessandrobartels - 01/02/2012
//     */
//    private function _atribuiBase13($base13){
//        /* @todo - Rever as validacoes.
//         * - Obrigatorio para a competencia 13.
//         * - Obrigatorio no mes de rescisao para quem trabalhou mais de 15 dias no ano e possui c�digo de
//         *   movimenta��o por motivo rescis�o (exceto rescis�o com justa causa), aposentadoria com quebra de
//         *   v�nculo ou falecimento.
//         * - Obrigat�rio para os c�digos de recolhimento 130 e 135
//         * � Obrigat�rio para os c�digos de recolhimento 608 quando houver trabalhador da categoria 02 no
//         *   arquivo.
//         * - Opcional para os c�digos de recolhimento 650.
//         * - So pode ser informado para as categorias 01, 02, 04, 06, 07, 12, 19, 20, 21 e 26.
//         * � Campo opcional para as categorias 01, 04, 06, 07, 12, 19, 20, 21 e 26 na compet�ncia 12.
//         * � Sempre que n�o informado preencher com zeros.
//         *
//         * PEGAR BASE DO ITEMCALCULO.
//         *
//         */
//        $codCategorias = array(
//            '01', '02', '04', '06', '07', '12', '19', '20', '21', '26',
//        );
//
//        $codRecol = array(
//            '130', '135', '650',
//        );
//
//        $codMovimentacoes = array(
//            'I1', 'I2', 'I3', 'I4', 'J', 'K', 'L',
//        );
//
//        if (array_search($this->_nrCategoria, $codCategorias) === false) {
//            $base13 = parent::CARACTER_ZERO;
//        } else {
//            if (($this->_competencia13 === false)
//                ||(array_search($this->_nrRecolhimento, $codRecol) === false)
//            ) {
//                /* verificar rescisao  - Movimentacoes*/
//                if ($this->_competencia === self::CARACTER_DOZE) {
//                    true;
//                    // Verifica competencia 12. Segundo ARTE deve passar o complemento do 13 se houver.
//                    /*  $base13 = //complemento de 13 salario se houver;*/
//                }
//            }
//        }
//
//        $this->atribuiDado($base13, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//    }
//
//
//    /**
//     * Atribui Base do 13 salario para GPS.
//     *
//     * @param string $base13GPS Valor da Base do 13 salario para GPS.
//     *
//     * @return void
//     *
//     * @author alessandrobartels - 02/02/2012
//     */
//    private function _atribuiBase13GPS($base13GPS){
//        /* @todo Verificar validacao.
//         * . Campo opcional para a compet�ncia 12.
//         * � N�o pode ser informado nas demais compet�ncias.
//         * � Campo opcional para as categorias 01, 04 , 06, 07, 12, 19, 20, 21 e 26.
//         * � Se informado o campo 22 (registro 40) deve ser diferente de zeros.
//         * � Sempre que n�o informado preencher com zeros.
//         *
//         *
//         * PEGAR BASE DO ITEMCALCULO.
//         * */
//
//        $codCategorias = array(
//            '01', '04', '06', '07', '12', '19', '20', '21', '26',
//        );
//
//        if ((array_search($this->_nrCategoria, $codCategorias) === false)
//            && (($this->_competencia !== self::CARACTER_DOZE)
//            ||($this->_competencia13 === true))
//        ) {
//            $base13GPS = parent::CARACTER_ZERO;
//        }
//
//        $this->atribuiDado($base13GPS, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//    }


}